@@ -26,7 +26,7 @@
  * This is mod_ifsession, contrib software for proftpd 1.2 and above.
  * For more information contact TJ Saunders <tj@castaglia.org>.
  *
- * $Id: mod_ifsession.c,v 1.6 2003/08/15 01:33:22 castaglia Exp $
+ * $Id: mod_ifsession.c,v 1.14 2004/07/08 21:38:24 castaglia Exp $
  */
 
 #include "conf.h"
@@ -34,8 +34,8 @@
 #define MOD_IFSESSION_VERSION	"mod_ifsession/0.9"
 
 /* Make sure the version of proftpd is as necessary. */
-#if PROFTPD_VERSION_NUMBER < 0x0001020901
-# error "ProFTPD 1.2.9rc1 or later required"
+#if PROFTPD_VERSION_NUMBER < 0x0001021001
+# error "ProFTPD 1.2.10rc1 or later required"
 #endif
 
 #define IFSESS_CLASS_NUMBER	100
@@ -45,9 +45,7 @@
 #define IFSESS_USER_NUMBER	102
 #define	IFSESS_USER_TEXT	"<IfUser>"
 
-#define IFSESS_EVAL_AND		201
-#define IFSESS_EVAL_OR		202
-#define	IFSESS_EVAL_REGEX	203
+static int ifsess_merged = FALSE;
 
 /* Support routines
  */
@@ -130,23 +128,6 @@ static void ifsess_dup_set(pool *dst_pool, xaset_t *dst, xaset_t *src) {
   }
 }
 
-/* Hooks
- */
-
-MODRET ifsess_merge(cmd_rec *cmd) {
-
-  /* This assumes that cmd->argv[0] contains a config_rec *, for the
-   * configuration to be merged in.
-   */
-  ifsess_dup_set(main_server->pool, main_server->conf,
-    ((config_rec *) cmd->argv[0])->subset);
-
-  resolve_deferred_dirs(main_server);
-  fixup_dirs(main_server, CF_DEFER);
-
-  return HANDLED(cmd);
-}
-
 /* Configuration handlers
  */
 
@@ -173,17 +154,17 @@ MODRET start_ifctxt(cmd_rec *cmd) {
   if (strcmp(cmd->argv[0], IFSESS_CLASS_TEXT) == 0) {
     name = "IfClassList";
     config_type = IFSESS_CLASS_NUMBER;
-    eval_type = IFSESS_EVAL_OR;
+    eval_type = PR_EXPR_EVAL_OR;
 
   } else if (strcmp(cmd->argv[0], IFSESS_GROUP_TEXT) == 0) {
     name = "IfGroupList";
     config_type = IFSESS_GROUP_NUMBER;
-    eval_type = IFSESS_EVAL_AND;
+    eval_type = PR_EXPR_EVAL_AND;
 
   } else if (strcmp(cmd->argv[0], IFSESS_USER_TEXT) == 0) {
     name = "IfUserList";
     config_type = IFSESS_USER_NUMBER;
-    eval_type = IFSESS_EVAL_OR;
+    eval_type = PR_EXPR_EVAL_OR;
   }
 
   /* Is this a normal expression, an explicit AND, an explicit OR, or a
@@ -191,12 +172,12 @@ MODRET start_ifctxt(cmd_rec *cmd) {
    */
   if (cmd->argc-1 > 1) {
     if (strcmp(cmd->argv[1], "AND") == 0) {
-      eval_type = IFSESS_EVAL_AND;
+      eval_type = PR_EXPR_EVAL_AND;
       argc = cmd->argc-2;
       argv = cmd->argv+1;
 
     } else if (strcmp(cmd->argv[1], "OR") == 0) {
-      eval_type = IFSESS_EVAL_OR;
+      eval_type = PR_EXPR_EVAL_OR;
       argc = cmd->argc-2;
       argv = cmd->argv+1;
 
@@ -223,7 +204,7 @@ MODRET start_ifctxt(cmd_rec *cmd) {
       c = add_config_param(name, 2, NULL, NULL);
       c->config_type = config_type;
       c->argv[0] = pcalloc(c->pool, sizeof(unsigned char));
-      *((unsigned char *) c->argv[0]) = IFSESS_EVAL_REGEX;
+      *((unsigned char *) c->argv[0]) = PR_EXPR_EVAL_REGEX;
       c->argv[1] = (void *) preg;
 
       return HANDLED(cmd);
@@ -244,7 +225,7 @@ MODRET start_ifctxt(cmd_rec *cmd) {
     argv = cmd->argv;
   }
 
-  acl = pr_parse_expression(cmd->tmp_pool, &argc, argv);
+  acl = pr_expr_create(cmd->tmp_pool, &argc, argv);
 
   c = add_config_param(name, 0);
 
@@ -279,6 +260,7 @@ MODRET end_ifctxt(cmd_rec *cmd) {
 MODRET ifsess_post_pass(cmd_rec *cmd) {
   register unsigned int i = 0;
   config_rec *c = NULL;
+  int found = 0;
   pool *tmp_pool = make_sub_pool(session.pool);
   array_header *group_remove_list = make_array(tmp_pool, 1,
     sizeof(config_rec *));
@@ -307,36 +289,36 @@ MODRET ifsess_post_pass(cmd_rec *cmd) {
       unsigned char mergein = FALSE;
 
 #if defined(HAVE_REGEX_H) && defined(HAVE_REGCOMP)
-      if (*((unsigned char *) list->argv[0]) == IFSESS_EVAL_REGEX) {
+      if (*((unsigned char *) list->argv[0]) == PR_EXPR_EVAL_REGEX) {
         regex_t *preg = (regex_t *) list->argv[1];
 
         if (session.group && regexec(preg, session.group, 0, NULL, 0) == 0)
           mergein = TRUE;
 
         else if (session.groups) {
-          register int i = 0;
+          register int j = 0;
 
-          for (i = session.groups->nelts-1; i >= 0; i--)
-            if (regexec(preg, *(((char **) session.groups->elts) + i), 0,
+          for (j = session.groups->nelts-1; j >= 0; j--)
+            if (regexec(preg, *(((char **) session.groups->elts) + j), 0,
                 NULL, 0) == 0)
               mergein = TRUE;
         }
       } else
 #endif /* HAVE_REGEX_H && HAVE_REGCOMP */
     
-      if (*((unsigned char *) list->argv[0]) == IFSESS_EVAL_OR &&
-          pr_group_or_expression((char **) &list->argv[1]))
+      if (*((unsigned char *) list->argv[0]) == PR_EXPR_EVAL_OR &&
+          pr_expr_eval_group_or((char **) &list->argv[1]))
         mergein = TRUE;
 
-      else if (*((unsigned char *) list->argv[0]) == IFSESS_EVAL_AND &&
-          pr_group_and_expression((char **) &list->argv[1]))
+      else if (*((unsigned char *) list->argv[0]) == PR_EXPR_EVAL_AND &&
+          pr_expr_eval_group_and((char **) &list->argv[1]))
         mergein = TRUE;
  
-      if (pr_group_and_expression((char **) &list->argv[0]))
+      if (pr_expr_eval_group_and((char **) &list->argv[0]))
         mergein = TRUE;
 
       if (mergein) {
-        log_debug(DEBUG2, MOD_IFSESSION_VERSION
+        pr_log_debug(DEBUG2, MOD_IFSESSION_VERSION
           ": merging <IfGroup> directives in");
         ifsess_dup_set(main_server->pool, main_server->conf, c->subset);
 
@@ -348,8 +330,10 @@ MODRET ifsess_post_pass(cmd_rec *cmd) {
         resolve_deferred_dirs(main_server);
         fixup_dirs(main_server, CF_DEFER);
 
+        ifsess_merged = TRUE;
+
       } else
-        log_debug(DEBUG9, MOD_IFSESSION_VERSION
+        pr_log_debug(DEBUG9, MOD_IFSESSION_VERSION
           ": <IfGroup> not matched, skipping");
     }
 
@@ -381,7 +365,7 @@ MODRET ifsess_post_pass(cmd_rec *cmd) {
       unsigned char mergein = FALSE;
 
 #if defined(HAVE_REGEX_H) && defined(HAVE_REGCOMP)
-      if (*((unsigned char *) list->argv[0]) == IFSESS_EVAL_REGEX) {
+      if (*((unsigned char *) list->argv[0]) == PR_EXPR_EVAL_REGEX) {
         regex_t *preg = (regex_t *) list->argv[1];
 
         if (regexec(preg, session.user, 0, NULL, 0) == 0)
@@ -390,16 +374,16 @@ MODRET ifsess_post_pass(cmd_rec *cmd) {
       } else
 #endif /* HAVE_REGEX_H && HAVE_REGCOMP */
 
-      if (*((unsigned char *) list->argv[0]) == IFSESS_EVAL_OR &&
-          pr_user_or_expression((char **) &list->argv[1]))
+      if (*((unsigned char *) list->argv[0]) == PR_EXPR_EVAL_OR &&
+          pr_expr_eval_user_or((char **) &list->argv[1]))
         mergein = TRUE;
 
-      else if (*((unsigned char *) list->argv[0]) == IFSESS_EVAL_AND &&
-          pr_user_and_expression((char **) &list->argv[1]))
+      else if (*((unsigned char *) list->argv[0]) == PR_EXPR_EVAL_AND &&
+          pr_expr_eval_user_and((char **) &list->argv[1]))
         mergein = TRUE;
 
       if (mergein) {
-        log_debug(DEBUG2, MOD_IFSESSION_VERSION
+        pr_log_debug(DEBUG2, MOD_IFSESSION_VERSION
           ": merging <IfUser> directives in");
         ifsess_dup_set(main_server->pool, main_server->conf, c->subset);
 
@@ -411,8 +395,10 @@ MODRET ifsess_post_pass(cmd_rec *cmd) {
         resolve_deferred_dirs(main_server);
         fixup_dirs(main_server, CF_DEFER);
 
+        ifsess_merged = TRUE;
+
       } else
-        log_debug(DEBUG9, MOD_IFSESSION_VERSION
+        pr_log_debug(DEBUG9, MOD_IFSESSION_VERSION
           ": <IfUser> not matched, skipping");
     }
 
@@ -426,6 +412,16 @@ MODRET ifsess_post_pass(cmd_rec *cmd) {
   }
 
   destroy_pool(tmp_pool);
+
+  if (ifsess_merged) {
+    /* Try to honor any <Limit LOGIN> sections that may have been merged in. */
+    if (!login_check_limits(TOPLEVEL_CONF, FALSE, TRUE, &found)) {
+      pr_log_auth(PR_LOG_NOTICE, "%s %s: Limit access denies login.",
+        session.anon_config ? "ANON" : C_USER, session.user);
+      end_login(0);
+    }
+  }
+
   return DECLINED(cmd);
 }
 
@@ -449,26 +445,26 @@ static int ifsess_sess_init(void) {
       unsigned char mergein = FALSE;
 
 #if defined(HAVE_REGEX_H) && defined(HAVE_REGCOMP)
-      if (*((unsigned char *) list->argv[0]) == IFSESS_EVAL_REGEX) {
+      if (*((unsigned char *) list->argv[0]) == PR_EXPR_EVAL_REGEX) {
         regex_t *preg = (regex_t *) list->argv[1];
 
-        if (session.class && regexec(preg, session.class->name, 0, NULL,
+        if (session.class && regexec(preg, session.class->cls_name, 0, NULL,
             0) == 0)
           mergein = TRUE;
 
       } else
 #endif /* HAVE_REGEX_H && HAVE_REGCOMP */
 
-      if (*((unsigned char *) list->argv[0]) == IFSESS_EVAL_OR &&
-          pr_class_or_expression((char **) &list->argv[1]))
+      if (*((unsigned char *) list->argv[0]) == PR_EXPR_EVAL_OR &&
+          pr_expr_eval_class_or((char **) &list->argv[1]))
         mergein = TRUE;
 
-      else if (*((unsigned char *) list->argv[0]) == IFSESS_EVAL_AND &&
-          pr_class_and_expression((char **) &list->argv[1]))
+      else if (*((unsigned char *) list->argv[0]) == PR_EXPR_EVAL_AND &&
+          pr_expr_eval_class_and((char **) &list->argv[1]))
         mergein = TRUE;
 
       if (mergein) {
-        log_debug(DEBUG2, MOD_IFSESSION_VERSION
+        pr_log_debug(DEBUG2, MOD_IFSESSION_VERSION
           ": merging <IfClass> directives in");
         ifsess_dup_set(main_server->pool, main_server->conf, c->subset);
 
@@ -480,8 +476,10 @@ static int ifsess_sess_init(void) {
         resolve_deferred_dirs(main_server);
         fixup_dirs(main_server, CF_DEFER);
 
+        ifsess_merged = TRUE;
+
       } else
-        log_debug(DEBUG9, MOD_IFSESSION_VERSION
+        pr_log_debug(DEBUG9, MOD_IFSESSION_VERSION
           ": <IfClass> not matched, skipping");
     }
 
@@ -513,10 +511,6 @@ static conftable ifsess_conftab[] = {
 
 static cmdtable ifsess_cmdtab[] = {
   { POST_CMD, C_PASS, G_NONE, ifsess_post_pass, FALSE, FALSE },
-
-  /* Hooks */
-  { HOOK, "ifsess_merge", G_NONE, ifsess_merge, FALSE, FALSE },
-
   { 0, NULL }
 };
 
