@@ -24,17 +24,17 @@
  * This is mod_rewrite, contrib software for proftpd 1.2 and above.
  * For more information contact TJ Saunders <tj@castaglia.org>.
  *
- * $Id: mod_rewrite.c,v 1.16 2003/08/06 22:03:32 castaglia Exp $
+ * $Id: mod_rewrite.c,v 1.19 2004/05/02 15:05:26 castaglia Exp $
  */
 
 #include "conf.h"
 #include "privs.h"
 
-#define MOD_REWRITE_VERSION "mod_rewrite/0.6.7"
+#define MOD_REWRITE_VERSION "mod_rewrite/0.6.8"
 
 /* Make sure the version of proftpd is as necessary. */
-#if PROFTPD_VERSION_NUMBER < 0x0001020801
-# error "ProFTPD 1.2.8rc1 or later required"
+#if PROFTPD_VERSION_NUMBER < 0x0001021001
+# error "ProFTPD 1.2.10rc1 or later required"
 #endif
 
 #ifdef HAVE_REGEX_H
@@ -156,7 +156,7 @@ static char *rewrite_expand_var(cmd_rec *cmd, const char *subst_pattern,
     const char *var) {
   if (!strcmp(var, "%c")) {
     REWRITE_CHECK_VAR(session.class, "%c");
-    return (session.class ? session.class->name : NULL);
+    return (session.class ? session.class->cls_name : NULL);
 
   } else if (!strcmp(var, "%F")) {
 
@@ -1197,6 +1197,52 @@ static int rewrite_utf8_to_ucs4(unsigned long *ucs4_buf,
  * probably will) modify their key arguments.
  */
 
+static char *rewrite_map_int_replaceall(pool *map_pool, char *key) {
+  char sep = *key;
+  char *value = NULL, *src = NULL, *dst = NULL;
+  char *tmp = NULL;
+
+  /* Due to the way in which this internal function works, the first
+   * character of the given key is used as a delimiter separating
+   * the given key, and the sequences to replace for this function.
+   */
+  char *str = pstrdup(map_pool, key + 1);
+  
+  tmp = strchr(str , sep);
+  if (tmp == NULL) {
+    rewrite_log("rewrite_map_int_replace(): badly formatted input key");
+    return NULL;
+  }
+
+  *tmp = '\0';
+  value = str;
+  rewrite_log("rewrite_map_int_replace(): actual key: '%s'", value); 
+ 
+  str = tmp + 1;
+
+  tmp = strchr(str, sep);
+  if (tmp == NULL) {
+    rewrite_log("rewrite_map_int_replace(): badly formatted input key");
+    return NULL;
+  }
+
+  *tmp = '\0';
+  src = str;
+  dst = tmp + 1;
+  
+  rewrite_log("rewrite_map_int_replace(): replacing '%s' with '%s'", src,
+    dst);
+
+  /* Make sure the source sequence is present in the given key. */
+  if (strstr(value, src) == NULL) {
+    rewrite_log("rewrite_map_int_replace(): '%s' does not occur in given "
+      "key '%s'", src, value);
+    return NULL;
+  }
+
+  return sreplace(map_pool, value, src, dst, NULL);
+}
+
 static char *rewrite_map_int_tolower(pool *map_pool, char *key) {
   register unsigned int i = 0;
   char *value = pstrdup(map_pool, key);
@@ -1421,12 +1467,12 @@ static void rewrite_openlog(void) {
 
   pr_signals_block();
   PRIVS_ROOT
-  res = log_openfile(rewrite_logfile, &rewrite_logfd, REWRITE_LOG_MODE);
+  res = pr_log_openfile(rewrite_logfile, &rewrite_logfd, REWRITE_LOG_MODE);
   PRIVS_RELINQUISH
   pr_signals_unblock();
 
   if (res < 0)
-    log_pri(LOG_NOTICE, MOD_REWRITE_VERSION
+    pr_log_pri(LOG_NOTICE, MOD_REWRITE_VERSION
       ": error: unable to open log file '%s': %s", rewrite_logfile,
       strerror(errno));
 
@@ -1440,8 +1486,8 @@ static void rewrite_closelog(void) {
     return;
 
   if (close(rewrite_logfd) < 0) {
-    log_pri(LOG_ERR, MOD_REWRITE_VERSION ": error closing RewriteLog '%s': %s",
-      rewrite_logfile, strerror(errno));
+    pr_log_pri(LOG_ERR, MOD_REWRITE_VERSION
+      ": error closing RewriteLog '%s': %s", rewrite_logfile, strerror(errno));
     return;
   }
 
@@ -1477,11 +1523,11 @@ static void rewrite_log(char *format, ...) {
 
   if (rewrite_logfd > 0) {
     if (write(rewrite_logfd, entry, strlen(entry)) < strlen(entry))
-      log_pri(LOG_ERR, "error writing to RewriteLog '%s': %s",
+      pr_log_pri(LOG_ERR, "error writing to RewriteLog '%s': %s",
         rewrite_logfile, strerror(errno));
 
   } else
-    log_debug(DEBUG3, MOD_REWRITE_VERSION ": %s", mesgbuf);
+    pr_log_debug(DEBUG3, MOD_REWRITE_VERSION ": %s", mesgbuf);
 
   return;
 }
@@ -1597,7 +1643,7 @@ MODRET set_rewritecondition(cmd_rec *cmd) {
       }
     }
     if (!is_valid_var)
-      log_debug(DEBUG1, "invalid RewriteCondition variable used");
+      pr_log_debug(DEBUG1, "invalid RewriteCondition variable used");
 
     var += 2;
   }
@@ -1708,18 +1754,20 @@ MODRET set_rewritemap(cmd_rec *cmd) {
   if (!strcmp(cmd->argv[2], "int")) {
     c = add_config_param(cmd->argv[0], 3, NULL, NULL, NULL);
 
-    /* Check that the given function is a valid internal mapping function
-     */
-    if (!strcmp(mapsrc, "tolower")) {
+    /* Check that the given function is a valid internal mapping function. */
+    if (strcmp(mapsrc, "replaceall") == 0) {
+      map = (void *) rewrite_map_int_replaceall;
+
+    } else if (strcmp(mapsrc, "tolower") == 0) {
       map = (void *) rewrite_map_int_tolower;
 
-    } else if (!strcmp(mapsrc, "toupper")) {
+    } else if (strcmp(mapsrc, "toupper") == 0) {
       map = (void *) rewrite_map_int_toupper;
 
-    } else if (!strcmp(mapsrc, "unescape")) {
+    } else if (strcmp(mapsrc, "unescape") == 0) {
       map = (void *) rewrite_map_int_unescape;
 
-    } else if (!strcmp(mapsrc, "utf8trans")) {
+    } else if (strcmp(mapsrc, "utf8trans") == 0) {
       map = (void *) rewrite_map_int_utf8trans;
 
     } else
@@ -1768,8 +1816,9 @@ MODRET set_rewritemap(cmd_rec *cmd) {
     txtmap->txt_path = pstrdup(txt_pool, mapsrc);    
 
     if (!rewrite_parse_map_txt(txtmap)) {
-      log_debug(DEBUG3, "%s: error parsing map file", cmd->argv[0]);
-      log_debug(DEBUG3, "%s: check the RewriteLog for details", cmd->argv[0]);
+      pr_log_debug(DEBUG3, "%s: error parsing map file", cmd->argv[0]);
+      pr_log_debug(DEBUG3, "%s: check the RewriteLog for details",
+        cmd->argv[0]);
     }
 
     map = (void *) txtmap;
@@ -2093,12 +2142,6 @@ static int rewrite_sess_init(void) {
 
 static int rewrite_init(void) {
 
-  /* Make sure the version of proftpd is as necessary. */
-  if (PROFTPD_VERSION_NUMBER < 0x0001020702) {
-    log_pri(LOG_ERR, MOD_REWRITE_VERSION " requires proftpd 1.2.7rc2 or later");
-    exit(1);
-  }
-
   /* Allocate a pool for this module's use. */
   if (!rewrite_pool)
     rewrite_pool = make_sub_pool(permanent_pool);
