@@ -1,5 +1,5 @@
 /*
- * ProFTPD: mod_postgres -- Support for connecting to Postgres databases.
+ * ProFTPD: mod_sql_postgres -- Support for connecting to Postgres databases.
  * Time-stamp: <1999-10-04 03:21:21 root>
  * Copyright (c) 2001 Andrew Houghton
  *  
@@ -21,6 +21,8 @@
  * holders give permission to link this program with OpenSSL, and distribute
  * the resulting executable, without including the source code for OpenSSL in
  * the source distribution.
+ *
+ * $Id: mod_sql_postgres.c,v 1.20 2004/05/08 03:21:46 castaglia Exp $
  */
 
 /*
@@ -31,11 +33,11 @@
  * Internal define used for debug and logging.  All backends are encouraged
  * to use the same format.
  */
-#define _MOD_VERSION "mod_sql_postgres/4.01"
+#define _MOD_VERSION "mod_sql_postgres/4.02"
 
 #define _POSTGRES_PORT "5432"
 
-#include <pgsql/libpq-fe.h>
+#include <libpq-fe.h>
 #include "conf.h"
 #include "../contrib/mod_sql.h"
 
@@ -92,8 +94,8 @@ typedef struct conn_entry_struct conn_entry_t;
 
 #define DEF_CONN_POOL_SIZE 10
 
-array_header *conn_cache;
-pool *conn_pool;
+pool *conn_pool = NULL;
+array_header *conn_cache = NULL;
 
 /*
  *  _sql_get_connection: walks the connection cache looking for the named
@@ -151,10 +153,9 @@ static void *_sql_add_connection(pool *p, char *name, db_conn_t *conn)
  *  properly filled in.  If not, it's grounds for the daemon to
  *  shutdown.
  */
-static void _sql_check_cmd(cmd_rec *cmd, char *msg)
-{
+static void _sql_check_cmd(cmd_rec *cmd, char *msg) {
   if ((!cmd) || (!cmd->tmp_pool)) {
-    log_pri(PR_LOG_ERR, _MOD_VERSION ": '%s' was passed an invalid cmd_rec. "
+    pr_log_pri(PR_LOG_ERR, _MOD_VERSION ": '%s' was passed an invalid cmd_rec. "
 	    "Shutting down.", msg);
     sql_log(DEBUG_WARN, "'%s' was passed an invalid cmd_rec. Shutting down.",
       msg);
@@ -169,8 +170,7 @@ static void _sql_check_cmd(cmd_rec *cmd, char *msg)
  *  that gets called.  This function makes assumptions about the 
  *  db_conn_t members.
  */
-static int _sql_timer_callback(CALLBACK_FRAME)
-{
+static int _sql_timer_callback(CALLBACK_FRAME) {
   conn_entry_t *entry = NULL;
   int cnt = 0;
   cmd_rec *cmd = NULL;
@@ -182,7 +182,7 @@ static int _sql_timer_callback(CALLBACK_FRAME)
       sql_log(DEBUG_INFO, "timer expired for connection '%s'", entry->name);
       cmd = _sql_make_cmd( conn_pool, 2, entry->name, "1" );
       cmd_close( cmd );
-      _sql_free_cmd( cmd );
+      SQL_FREE_CMD(cmd);
       entry->timer = 0;
     }
   }
@@ -190,23 +190,19 @@ static int _sql_timer_callback(CALLBACK_FRAME)
   return 0;
 }
 
-/* 
- * _sql_shutdown: walks the connection cache and closes every
+/* sql_postgres_exit_ev: walks the connection cache and closes every
  *  open connection, resetting their connection counts to 0.
  */
-static void _sql_shutdown(void)
-{
-  conn_entry_t *entry = NULL;
-  int cnt = 0;
-  cmd_rec *cmd;
+static void sql_postgres_exit_ev(const void *event_data, void *user_data) {
+  register unsigned int i = 0;
 
-  for (cnt=0; cnt < conn_cache->nelts; cnt++) {
-    entry = ((conn_entry_t **) conn_cache->elts)[cnt];
+  for (i = 0; i < conn_cache->nelts; i++) {
+    conn_entry_t *entry = ((conn_entry_t **) conn_cache->elts)[i];
 
     if (entry->connections > 0) {
-      cmd = _sql_make_cmd( conn_pool, 2, entry->name, "1" );
-      cmd_close( cmd );
-      _sql_free_cmd( cmd );
+      cmd_rec *cmd = _sql_make_cmd(conn_pool, 2, entry->name, "1");
+      cmd_close(cmd);
+      SQL_FREE_CMD(cmd);
     }
   }
 
@@ -677,7 +673,7 @@ MODRET cmd_select(cmd_rec *cmd)
 
     close_cmd = _sql_make_cmd( cmd->tmp_pool, 1, entry->name );
     cmd_close(close_cmd);
-    _sql_free_cmd( close_cmd );
+    SQL_FREE_CMD(close_cmd);
 
     sql_log(DEBUG_FUNC, "%s", "exiting \tpostgres cmd_select");
     return dmr;
@@ -695,7 +691,7 @@ MODRET cmd_select(cmd_rec *cmd)
 
     close_cmd = _sql_make_cmd( cmd->tmp_pool, 1, entry->name );
     cmd_close(close_cmd);
-    _sql_free_cmd( close_cmd );
+    SQL_FREE_CMD(close_cmd);
 
     return dmr;
   }    
@@ -703,7 +699,7 @@ MODRET cmd_select(cmd_rec *cmd)
   /* close the connection, return the data. */
   close_cmd = _sql_make_cmd( cmd->tmp_pool, 1, entry->name );
   cmd_close(close_cmd);
-  _sql_free_cmd( close_cmd );
+  SQL_FREE_CMD(close_cmd);
 
   sql_log(DEBUG_FUNC, "%s", "exiting \tpostgres cmd_select");
   return dmr;
@@ -796,7 +792,7 @@ MODRET cmd_insert(cmd_rec *cmd)
 
     close_cmd = _sql_make_cmd( cmd->tmp_pool, 1, entry->name );
     cmd_close(close_cmd);
-    _sql_free_cmd( close_cmd );
+    SQL_FREE_CMD(close_cmd);
 
     sql_log(DEBUG_FUNC, "%s", "exiting \tpostgres cmd_insert");
     return dmr;
@@ -807,7 +803,7 @@ MODRET cmd_insert(cmd_rec *cmd)
   /* close the connection and return HANDLED. */
   close_cmd = _sql_make_cmd( cmd->tmp_pool, 1, entry->name );
   cmd_close(close_cmd);
-  _sql_free_cmd( close_cmd );
+  SQL_FREE_CMD(close_cmd);
 
   sql_log(DEBUG_FUNC, "%s", "exiting \tpostgres cmd_insert");
   return HANDLED(cmd);
@@ -900,7 +896,7 @@ MODRET cmd_update(cmd_rec *cmd)
 
     close_cmd = _sql_make_cmd( cmd->tmp_pool, 1, entry->name );
     cmd_close(close_cmd);
-    _sql_free_cmd( close_cmd );
+    SQL_FREE_CMD(close_cmd);
 
     sql_log(DEBUG_FUNC, "%s", "exiting \tpostgres cmd_update");
     return dmr;
@@ -911,7 +907,7 @@ MODRET cmd_update(cmd_rec *cmd)
   /* close the connection, return HANDLED.  */
   close_cmd = _sql_make_cmd( cmd->tmp_pool, 1, entry->name );
   cmd_close(close_cmd);
-  _sql_free_cmd( close_cmd );
+  SQL_FREE_CMD(close_cmd);
 
   sql_log(DEBUG_FUNC, "%s", "exiting \tpostgres cmd_update");
   return HANDLED(cmd);
@@ -1022,7 +1018,7 @@ MODRET cmd_query(cmd_rec *cmd)
 
     close_cmd = _sql_make_cmd( cmd->tmp_pool, 1, entry->name );
     cmd_close(close_cmd);
-    _sql_free_cmd( close_cmd );
+    SQL_FREE_CMD(close_cmd);
 
     sql_log(DEBUG_FUNC, "%s", "exiting \tpostgres cmd_select");
     return dmr;
@@ -1047,7 +1043,7 @@ MODRET cmd_query(cmd_rec *cmd)
   /* close the connection, return the data. */
   close_cmd = _sql_make_cmd( cmd->tmp_pool, 1, entry->name );
   cmd_close(close_cmd);
-  _sql_free_cmd( close_cmd );
+  SQL_FREE_CMD(close_cmd);
 
   sql_log(DEBUG_FUNC, "%s", "exiting \tpostgres cmd_query");
   return dmr;
@@ -1075,13 +1071,15 @@ MODRET cmd_query(cmd_rec *cmd)
  *  the database documentation and figure it out) to do the conversion
  *  themselves in this function.
  *
- *  At the very least, a backend MUST simply copy the data from argv[0]
- *  into the data field of the modret.
+ *  A backend MUST supply a working escapestring implementation.  Simply
+ *  copying the data from argv[0] into the data field of the modret allows
+ *  for possible SQL injection attacks when this backend is used.
  */
-MODRET cmd_escapestring(cmd_rec * cmd)
-{
+MODRET cmd_escapestring(cmd_rec * cmd) {
   conn_entry_t *entry = NULL;
   db_conn_t *conn = NULL;
+  char *unescaped = NULL;
+  char *escaped = NULL;
 
   sql_log(DEBUG_FUNC, "%s", "entering \tpostgres cmd_escapestring");
 
@@ -1101,10 +1099,17 @@ MODRET cmd_escapestring(cmd_rec * cmd)
 
   conn = (db_conn_t *) entry->data;
 
-  /* PostgreSQL has no way to escape strings internally */
+  /* Note: the PQescapeString() function appeared in the C API as of
+   * Postgres-7.2.
+   */
+  unescaped = cmd->argv[1];
+  escaped = (char *) pcalloc(cmd->tmp_pool, sizeof(char) *
+    (strlen(unescaped) * 2) + 1);
+
+  PQescapeString(escaped, unescaped, strlen(unescaped));
 
   sql_log(DEBUG_FUNC, "%s", "exiting \tpostgres cmd_escapestring");
-  return mod_create_data(cmd, (void *) cmd->argv[1]);
+  return mod_create_data(cmd, (void *) escaped);
 }
 
 /*
@@ -1222,13 +1227,13 @@ cmdtable sql_cmdtable[] = {
  * sql_postgres_init: Used to initialize the connection cache and register
  *  the exit handler.
  */
-static int sql_postgres_init(void) {
-  conn_pool  = make_sub_pool(session.pool);
-  conn_cache = make_array(session.pool, DEF_CONN_POOL_SIZE,
-			  sizeof(conn_entry_t));
-
-  pr_exit_register_handler( _sql_shutdown );
+static int sql_postgres_sess_init(void) {
+  conn_pool = make_sub_pool(session.pool);
+  conn_cache = make_array(make_sub_pool(session.pool), DEF_CONN_POOL_SIZE,
+    sizeof(conn_entry_t));
 
+  pr_event_register(&sql_postgres_module, "core.exit", sql_postgres_exit_ev,
+    NULL);
   return 0;
 }
 
@@ -1239,12 +1244,27 @@ static int sql_postgres_init(void) {
  *  to extend the init functions to initialize other internal variables.
  */
 module sql_postgres_module = {
-  NULL, NULL,                   /* Always NULL */
-  0x20,                         /* API Version 2.0 */
+  /* Always NULL */
+  NULL, NULL,
+
+  /* Module API version */
+  0x20,
+
+  /* Module name */
   "sql_postgres",
-  NULL,                         /* SQL configuration handler table */
-  NULL,                         /* SQL command handler table */
-  NULL,                         /* SQL authentication handler table */
-  NULL,                         /* Pre-fork "daemon mode" init */
-  sql_postgres_init             /* Post-fork "child mode" init */
+
+  /* Module configuration directive handlers */
+  NULL,
+
+  /* Module command handlers */
+  NULL,
+
+  /* Module authentication handlers */
+  NULL,
+
+  /* Module initialization */
+  NULL,
+
+  /* Session initialization */
+  sql_postgres_sess_init
 };
