@@ -24,7 +24,7 @@
  * the source code for OpenSSL in the source distribution.
  */
 
-/* $Id: privs.h,v 1.21 2003/09/19 20:21:31 castaglia Exp $
+/* $Id: privs.h,v 1.27 2004/04/16 16:33:45 castaglia Exp $
  */
 
 #ifndef PR_PRIVS_H
@@ -57,9 +57,9 @@
  * doing this in here:
  */
 
-#ifdef __hpux
-# define setreuid(x,y) setresuid(x,y,0)
-#endif /* __hpux */
+#if defined(HPUX10) || defined(HPUX11)
+# define setreuid(r, e) setresuid((r), (e), 0)
+#endif /* HPUX */
 
 #ifdef PR_DEVEL_COREDUMP
 /* Unix kernels can be notoriously picky about dumping the core for
@@ -86,89 +86,92 @@
  */
 
 #  define PRIVS_SETUP(u, g) { \
-    log_debug(DEBUG9, "SETUP PRIVS at %s:%d", __FILE__, __LINE__); \
+    pr_log_debug(DEBUG9, "SETUP PRIVS at %s:%d", __FILE__, __LINE__); \
     if (getuid() != PR_ROOT_UID) { \
       session.ouid = session.uid = getuid(); \
       session.gid = getgid(); \
       if (setgid(session.gid)) \
-        log_pri(PR_LOG_ERR, "PRIVS_SETUP: unable to setgid(): %s", \
+        pr_log_pri(PR_LOG_ERR, "PRIVS_SETUP: unable to setgid(): %s", \
           strerror(errno)); \
       if (setreuid(session.uid, session.uid)) \
-        log_pri(PR_LOG_ERR, "PRIVS_SETUP: unable to setreuid(): %s", \
+        pr_log_pri(PR_LOG_ERR, "PRIVS_SETUP: unable to setreuid(): %s", \
           strerror(errno)); \
     } else {  \
       session.ouid = getuid(); \
       session.uid = (u); \
       session.gid = (g); \
       if (setgid(session.gid)) \
-        log_pri(PR_LOG_ERR, "PRIVS_SETUP: unable to setgid(): %s", \
+        pr_log_pri(PR_LOG_ERR, "PRIVS_SETUP: unable to setgid(): %s", \
           strerror(errno)); \
       if (setreuid(PR_ROOT_UID, session.uid)) \
-        log_pri(PR_LOG_ERR, "PRIVS_SETUP: unable to setreuid(): %s", \
+        pr_log_pri(PR_LOG_ERR, "PRIVS_SETUP: unable to setreuid(): %s", \
           strerror(errno)); \
     } \
   }
 
 #  define PRIVS_ROOT { \
-    log_debug(DEBUG9, "ROOT PRIVS at %s:%d", __FILE__, __LINE__); \
+    pr_log_debug(DEBUG9, "ROOT PRIVS at %s:%d", __FILE__, __LINE__); \
     if (!session.disable_id_switching) { \
-      if (setregid(session.gid, PR_ROOT_GID)) \
-        log_pri(PR_LOG_ERR, "PRIVS_ROOT: unable to setregid(): %s", \
-          strerror(errno)); \
       if (setreuid(session.uid, PR_ROOT_UID)) \
-        log_pri(PR_LOG_ERR, "PRIVS_ROOT: unable to setreuid(): %s", \
+        pr_log_pri(PR_LOG_ERR, "PRIVS_ROOT: unable to setreuid(): %s", \
+          strerror(errno)); \
+      if (setregid(session.gid, PR_ROOT_GID)) \
+        pr_log_pri(PR_LOG_ERR, "PRIVS_ROOT: unable to setregid(): %s", \
           strerror(errno)); \
     } else \
-      log_debug(DEBUG9, "ROOT PRIVS: ID switching disabled"); \
+      pr_log_debug(DEBUG9, "ROOT PRIVS: ID switching disabled"); \
   }
 
 #  define PRIVS_USER { \
-    log_debug(DEBUG9, "USER PRIVS %d at %s:%d", (int) session.login_uid, \
+    pr_log_debug(DEBUG9, "USER PRIVS %d at %s:%d", (int) session.login_uid, \
       __FILE__, __LINE__); \
     if (!session.disable_id_switching) { \
       if (setreuid(session.uid, PR_ROOT_UID)) \
-        log_pri(PR_LOG_ERR, "PRIVS_USER: unable to setreuid(session.uid, PR_ROOT_UID): %s", \
+        pr_log_pri(PR_LOG_ERR, "PRIVS_USER: unable to setreuid(session.uid, PR_ROOT_UID): %s", \
           strerror(errno)); \
+      if (setregid(session.gid, session.login_gid)) \
+        pr_log_pri(PR_LOG_ERR, "PRIVS_USER: unable to setregid(session.gid, " \
+          "session.login_gid): %s", strerror(errno)); \
       if (setreuid(session.uid, session.login_uid)) \
-        log_pri(PR_LOG_ERR, "PRIVS_USER: unable to setreuid(session.uid, " \
+        pr_log_pri(PR_LOG_ERR, "PRIVS_USER: unable to setreuid(session.uid, " \
           "session.login_uid): %s", strerror(errno)); \
     } else \
-      log_debug(DEBUG9, "ROOT PRIVS: ID switching disabled"); \
+      pr_log_debug(DEBUG9, "ROOT PRIVS: ID switching disabled"); \
   }
 
 #  define PRIVS_RELINQUISH  { \
-    log_debug(DEBUG9, "RELINQUISH PRIVS at %s:%d", __FILE__, __LINE__); \
+    pr_log_debug(DEBUG9, "RELINQUISH PRIVS at %s:%d", __FILE__, __LINE__); \
     if (!session.disable_id_switching) { \
       if (getegid() != PR_ROOT_GID) { \
         if (setregid(session.gid, PR_ROOT_GID)) \
-          log_pri(PR_LOG_ERR, "PRIVS_RELINQUISH: unable to " \
+          pr_log_pri(PR_LOG_ERR, "PRIVS_RELINQUISH: unable to " \
             "setregid(session.gid, PR_ROOT_GID): %s", strerror(errno)); \
       } \
       if (setregid(session.gid, session.gid)) \
-        log_pri(PR_LOG_ERR, "PRIVS_RELINQUISH: unable to setregid(session.jid, " \
+        pr_log_pri(PR_LOG_ERR, "PRIVS_RELINQUISH: unable to setregid(session.gid, " \
           "session.gid): %s", strerror(errno)); \
       if (geteuid() != PR_ROOT_UID) { \
         if (setreuid(session.uid, PR_ROOT_UID)) \
-          log_pri(PR_LOG_ERR, "PRIVS_RELINQUISH: unable to " \
+          pr_log_pri(PR_LOG_ERR, "PRIVS_RELINQUISH: unable to " \
             "setreuid(session.uid, PR_ROOT_UID): %s", strerror(errno)); \
       } \
       if (setreuid(session.uid, session.uid)) \
-        log_pri(PR_LOG_ERR, "PRIVS_RELINQUISH: unable to setreuid(session.uid, " \
+        pr_log_pri(PR_LOG_ERR, "PRIVS_RELINQUISH: unable to setreuid(session.uid, " \
           "session.uid): %s", strerror(errno)); \
     } else \
-      log_debug(DEBUG9, "ROOT PRIVS: ID switching disabled"); \
+      pr_log_debug(DEBUG9, "ROOT PRIVS: ID switching disabled"); \
   }
 
 #  define PRIVS_REVOKE { \
-    log_debug(DEBUG9, "REVOKE PRIVS at %s:%d", __FILE__, __LINE__); \
+    pr_log_debug(DEBUG9, "REVOKE PRIVS at %s:%d", __FILE__, __LINE__); \
     if (setreuid(PR_ROOT_UID, PR_ROOT_UID)) \
-      log_pri(PR_LOG_ERR, "PRIVS_REVOKE: unable to setreuid(PR_ROOT_UID, PR_ROOT_UID): %s", \
+      pr_log_pri(PR_LOG_ERR, "PRIVS_REVOKE: unable to setreuid(PR_ROOT_UID, PR_ROOT_UID): %s", \
         strerror(errno)); \
     if (setgid(session.gid)) \
-      log_pri(PR_LOG_ERR, "PRIVS_REVOKE: unable to setgid(): %s", \
+      pr_log_pri(PR_LOG_ERR, "PRIVS_REVOKE: unable to setgid(): %s", \
         strerror(errno)); \
     if (setuid(session.uid)) \
-      log_pri(PR_LOG_ERR, "PRIVS_REVOKE: unable to setuid(): %s", \
+      pr_log_pri(PR_LOG_ERR, "PRIVS_REVOKE: unable to setuid(): %s", \
         strerror(errno)); \
   }
 
@@ -185,31 +188,31 @@
  */
 
 #  define PRIVS_SETUP(u, g) { \
-    log_debug(DEBUG9, "SETUP PRIVS at %s:%d", __FILE__, __LINE__); \
+    pr_log_debug(DEBUG9, "SETUP PRIVS at %s:%d", __FILE__, __LINE__); \
     if (getuid() != PR_ROOT_UID) { \
       session.ouid = session.uid = getuid(); \
       session.gid = getgid(); \
       if (setgid(session.gid)) \
-        log_pri(PR_LOG_ERR, "PRIVS_SETUP: unable to setgid(): %s", \
+        pr_log_pri(PR_LOG_ERR, "PRIVS_SETUP: unable to setgid(): %s", \
           strerror(errno)); \
       if (setuid(session.uid)) \
-        log_pri(PR_LOG_ERR, "PRIVS_SETUP: unable to setuid(): %s", \
+        pr_log_pri(PR_LOG_ERR, "PRIVS_SETUP: unable to setuid(): %s", \
           strerror(errno)); \
       if (seteuid(session.uid)) \
-        log_pri(PR_LOG_ERR, "PRIVS_SETUP: unable to seteuid(): %s", \
+        pr_log_pri(PR_LOG_ERR, "PRIVS_SETUP: unable to seteuid(): %s", \
           strerror(errno)); \
     } else { \
       session.ouid = getuid(); \
       session.uid = (u); \
       session.gid = (g); \
       if (setuid(PR_ROOT_UID)) \
-        log_pri(PR_LOG_ERR, "PRIVS_SETUP: unable to setuid(): %s", \
+        pr_log_pri(PR_LOG_ERR, "PRIVS_SETUP: unable to setuid(): %s", \
           strerror(errno)); \
       if (setgid((g))) \
-        log_pri(PR_LOG_ERR, "PRIVS_SETUP: unable to setgid(): %s", \
+        pr_log_pri(PR_LOG_ERR, "PRIVS_SETUP: unable to setgid(): %s", \
           strerror(errno)); \
       if (seteuid((u))) \
-        log_pri(PR_LOG_ERR, "PRIVS_SETUP: unable to seteuid(): %s", \
+        pr_log_pri(PR_LOG_ERR, "PRIVS_SETUP: unable to seteuid(): %s", \
           strerror(errno)); \
     } \
   }
@@ -218,35 +221,33 @@
  */
 #  define PRIVS_ROOT \
   if (!session.disable_id_switching) { \
-    log_debug(DEBUG9, "ROOT PRIVS at %s:%d", __FILE__, __LINE__); \
+    pr_log_debug(DEBUG9, "ROOT PRIVS at %s:%d", __FILE__, __LINE__); \
     if (seteuid(PR_ROOT_UID)) \
-      log_pri(PR_LOG_ERR, "PRIVS_ROOT: unable to seteuid(): %s", \
+      pr_log_pri(PR_LOG_ERR, "PRIVS_ROOT: unable to seteuid(): %s", \
         strerror(errno)); \
     if (setegid(PR_ROOT_GID)) \
-      log_pri(PR_LOG_ERR, "PRIVS_ROOT: unable to setegid(): %s", \
+      pr_log_pri(PR_LOG_ERR, "PRIVS_ROOT: unable to setegid(): %s", \
         strerror(errno)); \
   } else \
-    log_debug(DEBUG9, "ROOT PRIVS: ID switching disabled");
+    pr_log_debug(DEBUG9, "ROOT PRIVS: ID switching disabled");
 
 /* Switch to the privs of the login user.
  */
 #  define PRIVS_USER \
   if (!session.disable_id_switching) { \
-    if (session.login_uid == PR_ROOT_UID) { \
-      log_debug(DEBUG1, "Use of PRIVS_USER before session.login_uid set " \
-        "in %s %d", __FILE__, __LINE__); \
-    } else { \
-      log_debug(DEBUG9, "USER PRIVS %d at %s:%d", (int) session.login_uid, \
-        __FILE__, __LINE__); \
-      if (seteuid(PR_ROOT_UID)) \
-        log_pri(PR_LOG_ERR, "PRIVS_USER: unable to seteuid(PR_ROOT_UID): %s", \
-          strerror(errno)); \
-      if (seteuid(session.login_uid)) \
-        log_pri(PR_LOG_ERR, "PRIVS_USER: unable to seteuid(session.login_uid): " \
-          "%s", strerror(errno)); \
-    } \
+    pr_log_debug(DEBUG9, "USER PRIVS %d at %s:%d", (int) session.login_uid, \
+      __FILE__, __LINE__); \
+    if (seteuid(PR_ROOT_UID)) \
+      pr_log_pri(PR_LOG_ERR, "PRIVS_USER: unable to seteuid(PR_ROOT_UID): %s", \
+        strerror(errno)); \
+    if (setegid(session.login_gid)) \
+      pr_log_pri(PR_LOG_ERR, "PRIVS_USER: unable to setegid(session.login_gid): " \
+        "%s", strerror(errno)); \
+    if (seteuid(session.login_uid)) \
+      pr_log_pri(PR_LOG_ERR, "PRIVS_USER: unable to seteuid(session.login_uid): " \
+        "%s", strerror(errno)); \
   } else \
-    log_debug(DEBUG9, "ROOT PRIVS: ID switching disabled");
+    pr_log_debug(DEBUG9, "ROOT PRIVS: ID switching disabled");
 
 /* Relinquish privs granted by PRIVS_ROOT or PRIVS_USER.
  */
@@ -254,31 +255,31 @@
   if (!session.disable_id_switching) { \
     if (geteuid() != PR_ROOT_UID) { \
       if (seteuid(PR_ROOT_UID)) \
-        log_pri(PR_LOG_ERR, "PRIVS_RELINQUISH: unable to seteuid(PR_ROOT_UID): %s", \
+        pr_log_pri(PR_LOG_ERR, "PRIVS_RELINQUISH: unable to seteuid(PR_ROOT_UID): %s", \
           strerror(errno)); \
     } \
-    log_debug(DEBUG9, "RELINQUISH PRIVS at %s:%d", __FILE__, __LINE__); \
+    pr_log_debug(DEBUG9, "RELINQUISH PRIVS at %s:%d", __FILE__, __LINE__); \
     if (setegid(session.gid)) \
-      log_pri(PR_LOG_ERR, "PRIVS_RELINQUISH: unable to setegid(session.gid): %s", \
+      pr_log_pri(PR_LOG_ERR, "PRIVS_RELINQUISH: unable to setegid(session.gid): %s", \
         strerror(errno)); \
     if (seteuid(session.uid)) \
-      log_pri(PR_LOG_ERR, "PRIVS_RELINQUISH: unable to seteuid(session.uid): %s", \
+      pr_log_pri(PR_LOG_ERR, "PRIVS_RELINQUISH: unable to seteuid(session.uid): %s", \
         strerror(errno)); \
   } else \
-    log_debug(DEBUG9, "ROOT PRIVS: ID switching disabled");
+    pr_log_debug(DEBUG9, "ROOT PRIVS: ID switching disabled");
 
 /* Revoke all privs.
  */
 #  define PRIVS_REVOKE { \
-    log_debug(DEBUG9, "REVOKE PRIVS at %s:%d", __FILE__, __LINE__); \
+    pr_log_debug(DEBUG9, "REVOKE PRIVS at %s:%d", __FILE__, __LINE__); \
     if (seteuid(PR_ROOT_UID)) \
-      log_pri(PR_LOG_ERR, "PRIVS_REVOKE: unable to seteuid(): %s", \
+      pr_log_pri(PR_LOG_ERR, "PRIVS_REVOKE: unable to seteuid(): %s", \
         strerror(errno)); \
     if (setgid(session.gid)) \
-      log_pri(PR_LOG_ERR, "PRIVS_REVOKE: unable to setgid(): %s", \
+      pr_log_pri(PR_LOG_ERR, "PRIVS_REVOKE: unable to setgid(): %s", \
         strerror(errno)); \
     if (setuid(session.uid)) \
-      log_pri(PR_LOG_ERR, "PRIVS_REVOKE: unable to setuid(): %s", \
+      pr_log_pri(PR_LOG_ERR, "PRIVS_REVOKE: unable to setuid(): %s", \
         strerror(errno)); \
   }
 
