@@ -25,12 +25,14 @@
  */
 
 /* Non-specific support functions.
- * $Id: support.h,v 1.18 2003/01/16 02:04:46 castaglia Exp $
+ * $Id: support.h,v 1.23 2004/06/07 22:58:44 castaglia Exp $
  */
 
 #ifndef PR_SUPPORT_H
 #define PR_SUPPORT_H
 
+#include <time.h>
+
 #define CHOP(s)		strip_end((s),"\r\n")
 
 #if defined(NAME_MAX)
@@ -87,14 +89,19 @@ int exists(char *);
 int access_check(char *, int);
 char *make_arg_str(pool *, int, char **);
 
+char *pr_str_strip(pool *, char *);
 char *strip_end(char *, char *);
 char *get_token(char **, char *);
 char *safe_token(char **);
 int check_shutmsg(time_t *, time_t *, time_t *, char *, size_t);
 
 void pr_memscrub(void *, size_t);
+
+struct tm *pr_gmtime(pool *, const time_t *);
+struct tm *pr_localtime(pool *, const time_t *);
+const char *pr_strtime(time_t);
+
 char *sstrcat(char *, const char *, size_t);
-char *sstrncpy(char *, const char *, size_t);
 char *sreplace(pool *, char *, ...);
 
 #endif /* PR_SUPPORT_H */
