@@ -23,17 +23,18 @@ not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite
 
 #include <conf.h>
 
-/* From log.c */
-extern void log_pri(int, char *, ...);
+/* From log.c/log.h */
+#define PR_LOG_ERR LOG_ERR
+extern void pr_log_pri(int, char *, ...);
 
-/* From utils.c */
+/* From support.c */
 extern char *sstrncpy(char *dest, const char *src, size_t n);
 
 #define NPWDFIELDS 	7
 #define NGRPFIELDS 	4
 
 #ifndef BUFSIZ
-#define BUFSIZ		TUNABLE_BUFFER_SIZE
+#define BUFSIZ		PR_TUNABLE_BUFFER_SIZE
 #endif
 
 /* Provides fgetpwent()/fgetgrent() functions.  Note that the format of the
@@ -92,7 +93,7 @@ static struct passwd *supp_getpwent(const char *buf) {
   pwd->pw_dir = fields[5];
   pwd->pw_shell = fields[6];
 
-  return pwd;  
+  return pwd;
 }
 
 struct passwd *fgetpwent(FILE *fp) {
@@ -132,7 +133,7 @@ static char *fgetbufline(char **buf, int *buflen, FILE *fp) {
     /* No -- allocate a larger buffer, doubling buflen. */
     *buflen += *buflen;
 
-    { 
+    {
       char *new_buf;
 
       if ((new_buf = realloc(*buf, *buflen)) == NULL)
@@ -173,17 +174,17 @@ static struct group *supp_getgrent(const char *buf) {
   char *cp;
 
   i = strlen(buf) + 1;
-  
+
   if (!grpbuf)
     grpbuf = malloc(i);
   else
     grpbuf = realloc(grpbuf, i);
-  
+
   if (!grpbuf)
     return NULL;
-  
+
   sstrncpy(grpbuf, buf, i);
-  
+
   if ((cp = strrchr(grpbuf, '\n')))
     *cp = '\0';
 
@@ -195,10 +196,10 @@ static struct group *supp_getgrent(const char *buf) {
   }
 
   if (i < (NGRPFIELDS - 1)) {
-    log_pri(LOG_ERR, "Malformed entry in group file: %s", buf);
+    pr_log_pri(PR_LOG_ERR, "Malformed entry in group file: %s", buf);
     return NULL;
   }
-  
+
   if (*grpfields[2] == '\0')
     return NULL;
 
