@@ -25,7 +25,7 @@
 
 /*
  * "SITE" commands module for ProFTPD
- * $Id: mod_site.c,v 1.38 2003/08/01 01:05:25 castaglia Exp $
+ * $Id: mod_site.c,v 1.41 2004/02/23 23:27:15 castaglia Exp $
  */
 
 #include "conf.h"
@@ -86,7 +86,7 @@ MODRET site_chgrp(cmd_rec *cmd) {
   preg = (regex_t *) get_param_ptr(CURRENT_CONF, "PathAllowFilter", FALSE);
 
   if (preg && regexec(preg, arg, 0, NULL, 0) != 0) {
-    log_debug(DEBUG2, "'%s %s' denied by PathAllowFilter", cmd->argv[0],
+    pr_log_debug(DEBUG2, "'%s %s' denied by PathAllowFilter", cmd->argv[0],
       cmd->arg);
     pr_response_add_err(R_550, "%s: Forbidden filename", arg);
     return ERROR(cmd);
@@ -95,7 +95,7 @@ MODRET site_chgrp(cmd_rec *cmd) {
   preg = (regex_t *) get_param_ptr(CURRENT_CONF, "PathDenyFilter", FALSE);
 
   if (preg && regexec(preg, arg, 0, NULL, 0) == 0) {
-    log_debug(DEBUG2, "'%s %s' denied by PathDenyFilter", cmd->argv[0],
+    pr_log_debug(DEBUG2, "'%s %s' denied by PathDenyFilter", cmd->argv[0],
       cmd->arg);
     pr_response_add_err(R_550, "%s: Forbidden filename", arg);
     return ERROR(cmd);
@@ -156,7 +156,7 @@ MODRET site_chmod(cmd_rec *cmd) {
   preg = (regex_t *) get_param_ptr(CURRENT_CONF, "PathAllowFilter", FALSE);
 
   if (preg && regexec(preg, arg, 0, NULL, 0) != 0) {
-    log_debug(DEBUG2, "'%s %s %s' denied by PathAllowFilter", cmd->argv[0],
+    pr_log_debug(DEBUG2, "'%s %s %s' denied by PathAllowFilter", cmd->argv[0],
       cmd->argv[1], arg);
     pr_response_add_err(R_550, "%s: Forbidden filename", arg);
     return ERROR(cmd);
@@ -165,7 +165,7 @@ MODRET site_chmod(cmd_rec *cmd) {
   preg = (regex_t *) get_param_ptr(CURRENT_CONF, "PathDenyFilter", FALSE);
 
   if (preg && regexec(preg, arg, 0, NULL, 0) == 0) {
-    log_debug(DEBUG2, "'%s %s %s' denied by PathDenyFilter", cmd->argv[0],
+    pr_log_debug(DEBUG2, "'%s %s %s' denied by PathDenyFilter", cmd->argv[0],
       cmd->argv[1], arg);
     pr_response_add_err(R_550, "%s: Forbidden filename", arg);
     return ERROR(cmd);
@@ -459,12 +459,19 @@ MODRET site_post_cmd(cmd_rec *cmd) {
   return DECLINED(cmd);
 }
 
-/* Module API tables
+/* Initialization routines
  */
 
-static conftable site_conftab[] = {
-  { NULL, 		NULL,			NULL }
-};
+static int site_init(void) {
+
+  /* Add the commands handled by this module to the HELP list. */ 
+  pr_help_add(C_SITE, "<sp> string", TRUE);
+
+  return 0;
+}
+
+/* Module API tables
+ */
 
 static cmdtable site_cmdtab[] = {
   { PRE_CMD,  C_SITE, G_NONE, site_pre_cmd,   FALSE,  FALSE },
@@ -483,7 +490,7 @@ module site_module = {
   "site",
 
   /* Module configuration table */
-  site_conftab,
+  NULL,
 
   /* Module command handler table */
   site_cmdtab,
@@ -492,7 +499,7 @@ module site_module = {
   NULL,
 
   /* Module initialization function */
-  NULL,
+  site_init,
 
   /* Session initialization function */
   NULL
