@@ -26,7 +26,7 @@
 
 /*
  * Data connection management functions
- * $Id: data.c,v 1.70 2003/10/06 03:53:49 castaglia Exp $
+ * $Id: data.c,v 1.79 2004/08/25 18:08:46 castaglia Exp $
  */
 
 #include "conf.h"
@@ -51,10 +51,13 @@
  */
 static pr_netio_stream_t *nstrm = NULL;
 
+static long timeout_linger = PR_TUNABLE_TIMEOUTLINGER;
+
 /* Called if the "Stalled" timer goes off
  */
 static int stalled_timeout_cb(CALLBACK_FRAME) {
-  log_pri(PR_LOG_NOTICE, "Data transfer stall timeout: %d seconds",
+  pr_event_generate("core.timeout-stalled", NULL);
+  pr_log_pri(PR_LOG_NOTICE, "Data transfer stall timeout: %d seconds",
     TimeoutStalled);
   end_login(1);
 
@@ -112,6 +115,8 @@ static int xlate_ascii_read(char *buf, int *bufsize, int *adjlen) {
  *           and should be the difference between buflen's original
  *           value and its value when this function returns
  */
+
+static int have_dangling_cr = FALSE;
 static unsigned int xlate_ascii_write(char **buf, unsigned int *buflen,
     unsigned int bufsize) {
   char *tmpbuf = *buf;
@@ -123,13 +128,20 @@ static unsigned int xlate_ascii_write(char **buf, unsigned int *buflen,
   register unsigned int i = 0;
 
   /* First, determine how many bare LFs are present. */
-  if (tmpbuf[0] == '\n')
+  if (!have_dangling_cr && tmpbuf[0] == '\n')
     lfcount++;
 
   for (i = 1; i < tmplen; i++)
     if (tmpbuf[i] == '\n' && tmpbuf[i-1] != '\r')
       lfcount++;
 
+  /* If the last character in the buffer is CR, then we have a dangling CR.
+   * The first character in the next buffer could be an LF, and without
+   * this flag, that LF would be treated as a bare LF, thus resulting in
+   * an added extraneous CR in the stream.
+   */
+  have_dangling_cr = (tmpbuf[tmplen-1] == '\r') ? TRUE : FALSE;
+
   if (lfcount == 0)
     /* No translation needed. */
     return 0;
@@ -149,17 +161,22 @@ static unsigned int xlate_ascii_write(char **buf, unsigned int *buflen,
    * never be less than zero).
    */
   if ((res = (bufsize - tmplen - lfcount)) <= 0) {
-    pool *copy_pool = make_sub_pool(session.xfer.p);
-    char *copy_buf = pcalloc(copy_pool, tmplen);
+    char *copybuf = malloc(tmplen);
+    if (!copybuf) {
+      pr_log_pri(PR_LOG_ERR, "fatal: memory exhausted");
+      exit(1);
+    }
 
-    memcpy(copy_buf, tmpbuf, tmplen);
+    memcpy(copybuf, tmpbuf, tmplen);
 
     /* Allocate a new session.xfer.buf of the needed size. */
     session.xfer.bufsize = tmplen + lfcount + 1;
     session.xfer.buf = pcalloc(session.xfer.p, session.xfer.bufsize);
 
-    memcpy(session.xfer.buf, copy_buf, tmplen);
-    destroy_pool(copy_pool);
+    memcpy(session.xfer.buf, copybuf, tmplen);
+
+    free(copybuf);
+    copybuf = NULL;
 
     tmpbuf = session.xfer.buf;
     bufsize = session.xfer.bufsize;
@@ -202,7 +219,9 @@ static void data_new_xfer(char *filename, int direction) {
   }
 
   session.xfer.p = make_sub_pool(session.pool);
-  session.xfer.filename = pstrdup(session.xfer.p,filename);
+  pr_pool_tag(session.xfer.p, "session.xfer pool");
+
+  session.xfer.filename = pstrdup(session.xfer.p, filename);
   session.xfer.direction = direction;
   session.xfer.bufsize = PR_TUNABLE_BUFFER_SIZE;
   session.xfer.buf = pcalloc(session.xfer.p, PR_TUNABLE_BUFFER_SIZE + 1);
@@ -252,9 +271,9 @@ static int data_pasv_open(char *reason, off_t size) {
     pr_inet_set_nonblock(session.pool, c);
     session.d = c;
 
-    log_debug(DEBUG4, "passive data connection opened - local  : %s:%d",
+    pr_log_debug(DEBUG4, "passive data connection opened - local  : %s:%d",
       pr_netaddr_get_ipstr(session.d->local_addr), session.d->local_port);
-    log_debug(DEBUG4, "passive data connection opened - remote : %s:%d",
+    pr_log_debug(DEBUG4, "passive data connection opened - remote : %s:%d",
       pr_netaddr_get_ipstr(session.d->remote_addr), session.d->remote_port);
 
     if (session.xfer.xfer_type != STOR_UNIQUE) {
@@ -294,7 +313,7 @@ static int data_pasv_open(char *reason, off_t size) {
 
   /* Check for error conditions. */
   if (c && c->mode == CM_ERROR)
-    log_pri(PR_LOG_ERR, "Error: unable to accept an incoming data "
+    pr_log_pri(PR_LOG_ERR, "Error: unable to accept an incoming data "
       "connection (%s)", strerror(c->xerrno));
 
   pr_response_add_err(R_425, "Unable to build data connection: %s",
@@ -352,9 +371,9 @@ static int data_active_open(char *reason, off_t size) {
   pr_netaddr_set_reverse_dns(rev);
 
   if (c) {
-    log_debug(DEBUG4, "active data connection opened - local  : %s:%d",
+    pr_log_debug(DEBUG4, "active data connection opened - local  : %s:%d",
       pr_netaddr_get_ipstr(session.d->local_addr), session.d->local_port);
-    log_debug(DEBUG4, "active data connection opened - remote : %s:%d",
+    pr_log_debug(DEBUG4, "active data connection opened - remote : %s:%d",
       pr_netaddr_get_ipstr(session.d->remote_addr), session.d->remote_port);
 
     if (session.xfer.xfer_type != STOR_UNIQUE) {
@@ -402,6 +421,10 @@ static int data_active_open(char *reason, off_t size) {
   return -1;
 }
 
+void pr_data_set_linger(long linger) {
+  timeout_linger = linger;
+}
+
 void pr_data_reset(void) {
   if (session.d && session.d->pool)
     destroy_pool(session.d->pool);
@@ -415,7 +438,7 @@ void pr_data_init(char *filename, int direction) {
 
   } else {
     if (!(session.sf_flags & SF_PASSIVE))
-      log_debug(DEBUG0,
+      pr_log_debug(DEBUG0,
 		"data_init oddity: session.xfer exists in non-PASV mode.");
 
     session.xfer.direction = direction;
@@ -434,9 +457,10 @@ int pr_data_open(char *filename, char *reason, int direction, off_t size) {
     reason = filename;
 
   /* Passive data transfers... */
-  if (session.sf_flags & SF_PASSIVE) {
+  if (session.sf_flags & SF_PASSIVE ||
+      session.sf_flags & SF_EPSV_ALL) {
     if (!session.d) {
-      log_pri(PR_LOG_ERR, "Internal error: PASV mode set, but no data "
+      pr_log_pri(PR_LOG_ERR, "Internal error: PASV mode set, but no data "
         "connection listening.");
       end_login(1);
     }
@@ -446,7 +470,7 @@ int pr_data_open(char *filename, char *reason, int direction, off_t size) {
   /* Active data transfers... */
   } else {
     if (session.d) {
-      log_pri(PR_LOG_ERR, "Internal error: non-PASV mode, yet data "
+      pr_log_pri(PR_LOG_ERR, "Internal error: non-PASV mode, yet data "
         "connection already exists?!?");
       end_login(1);
     }
@@ -514,7 +538,7 @@ void pr_data_close(int quiet) {
   nstrm = NULL;
 
   if (session.d) {
-    pr_inet_lingering_close(session.pool, session.d, PR_TUNABLE_TIMEOUTLINGER);
+    pr_inet_lingering_close(session.pool, session.d, timeout_linger);
     session.d = NULL;
   }
 
@@ -547,7 +571,7 @@ void pr_data_close(int quiet) {
 void pr_data_cleanup(void) {
   /* sanity check */
   if (session.d) {
-    pr_inet_lingering_close(session.pool, session.d, PR_TUNABLE_TIMEOUTLINGER);
+    pr_inet_lingering_close(session.pool, session.d, timeout_linger);
     session.d = NULL;
   }
 
@@ -566,7 +590,7 @@ void pr_data_abort(int err, int quiet) {
   nstrm = NULL;
 
   if (session.d) {
-    pr_inet_lingering_close(session.pool, session.d, PR_TUNABLE_TIMEOUTLINGER);
+    pr_inet_lingering_close(session.pool, session.d, timeout_linger);
     session.d = NULL;
   }
 
@@ -874,7 +898,7 @@ int pr_data_xfer(char *cl_buf, int cl_size) {
   }
 
   if (total && TimeoutIdle)
-    reset_timer(TIMER_IDLE,ANY_MODULE);
+    reset_timer(TIMER_IDLE, ANY_MODULE);
 
   session.xfer.total_bytes += total;
   session.total_bytes += total;
@@ -923,13 +947,16 @@ pr_sendfile_t pr_data_sendfile(int retr_fd, off_t *offset, size_t count) {
        */
 
       count -= len;
-	
-      if (TimeoutStalled)
-        reset_timer(TIMER_STALLED, ANY_MODULE);
-	
-      if (TimeoutIdle)
-	reset_timer(TIMER_IDLE, ANY_MODULE);
-	
+
+      /* Only reset the timers if data have actually been written out. */
+      if (len > 0) {
+        if (TimeoutStalled)
+          reset_timer(TIMER_STALLED, ANY_MODULE);
+
+        if (TimeoutIdle)
+          reset_timer(TIMER_IDLE, ANY_MODULE);
+      }
+
       session.xfer.total_bytes += len;
       session.total_bytes += len;
       total += len;
