@@ -22,9 +22,8 @@
  * OpenSSL in the source distribution.
  */
 
-/*
- * Home-on-demand support
- * $Id: mkhome.c,v 1.2 2003/03/26 03:34:09 castaglia Exp $
+/* Home-on-demand support
+ * $Id: mkhome.c,v 1.6 2004/05/13 01:13:57 castaglia Exp $
  */
 
 #include "conf.h"
@@ -40,14 +39,14 @@ static int create_dir(const char *dir, uid_t uid, gid_t gid,
   res = pr_fsio_stat(dir, &st);
 
   if (res == -1 && errno != ENOENT) {
-    log_pri(PR_LOG_WARNING, "error checking '%s': %s", dir,
+    pr_log_pri(PR_LOG_WARNING, "error checking '%s': %s", dir,
       strerror(errno));
     return -1;
   }
 
   /* The directory already exists. */
   if (res == 0) {
-    log_debug(DEBUG3, "CreateHome: '%s' already exists", dir);
+    pr_log_debug(DEBUG3, "CreateHome: '%s' already exists", dir);
     return 0;
   }
 
@@ -55,20 +54,20 @@ static int create_dir(const char *dir, uid_t uid, gid_t gid,
   prevmask = umask(0);
 
   if (pr_fsio_mkdir(dir, mode) < 0) {
-    log_pri(PR_LOG_WARNING, "error creating '%s': %s", dir,
+    pr_log_pri(PR_LOG_WARNING, "error creating '%s': %s", dir,
       strerror(errno));
     return -1;
   }
 
   if (pr_fsio_chown(dir, uid, gid) < 0) {
-    log_pri(PR_LOG_WARNING, "error setting ownership of '%s': %s", dir,
+    pr_log_pri(PR_LOG_WARNING, "error setting ownership of '%s': %s", dir,
       strerror(errno));
     return -1;
   }
 
   umask(prevmask);
 
-  log_debug(DEBUG6, "CreateHome: directory '%s' created", dir);
+  pr_log_debug(DEBUG6, "CreateHome: directory '%s' created", dir);
   return 0;
 }
 
@@ -81,11 +80,14 @@ static int create_path(pool *p, const char *path, const char *user, uid_t uid,
   struct stat st;
 
   pr_fs_clear_cache();
-  if (pr_fsio_stat(path, &st) == 0)
+  if (pr_fsio_stat(path, &st) == 0) {
     /* Path already exists, nothing to be done. */
-    return 0;
+    errno = EEXIST;
+    return -1;
+  }
 
-  log_debug(DEBUG3, "creating home directory '%s' for user '%s'", path, user);
+  pr_log_debug(DEBUG3, "creating home directory '%s' for user '%s'", path,
+    user);
   tmppath = pstrdup(p, path);
 
   currpath = "/";
@@ -104,56 +106,7 @@ static int create_path(pool *p, const char *path, const char *user, uid_t uid,
     pr_signals_handle();
   }
 
-  log_debug(DEBUG3, "home directory '%s' created", path);
-  return 0;
-}
-
-static int copy_file(pool *p, const char *src, const char *dst, uid_t uid,
-    gid_t gid, mode_t mode) {
-  pr_fh_t *src_fh = NULL, *dst_fh = NULL;
-  char buf[PR_TUNABLE_BUFFER_SIZE] = {'\0'};
-  int res = 0;
-
-  if ((src_fh = pr_fsio_open(src, O_RDONLY)) == NULL) {
-    log_debug(DEBUG2, "CreateHome: trouble with '%s': %s", src, 
-      strerror(errno));
-    return -1;
-  }
-
-  if ((dst_fh = pr_fsio_open(dst, O_WRONLY|O_CREAT|O_EXCL)) == NULL) {
-    log_debug(DEBUG2, "CreateHome: trouble with %s': %s", dst, 
-      strerror(errno));
-    pr_fsio_close(src_fh);
-    return -1;
-  }
-
-  /* Make sure the destination file starts with a zero size. */
-  pr_fsio_truncate(dst, 0);
-
-  while ((res = pr_fsio_read(src_fh, buf, sizeof(buf))) > 0) {
-    if (pr_fsio_write(dst_fh, buf, res) != res) {
-      log_pri(PR_LOG_WARNING, "CreateHome: error writing to '%s': %s",
-        dst, strerror(errno));
-      break;
-    }
-
-    pr_signals_handle();
-  }
-
-  /* Make sure the destination file has the proper ownership and mode. */
-  if (pr_fsio_chown(dst, uid, gid) < 0)
-    log_pri(PR_LOG_WARNING, "CreateHome: error chown'ing '%s' to %u/%u: %s",
-      dst, (unsigned int) uid, (unsigned int) gid, strerror(errno));
- 
-  if (pr_fsio_chmod(dst, mode) < 0)
-    log_pri(PR_LOG_WARNING, "CreateHome: error chmod'ing '%s' to %04o: %s",
-      dst, (unsigned int) mode, strerror(errno));
-
-  pr_fsio_close(src_fh);
-  if (pr_fsio_close(dst_fh) < 0)
-    log_pri(PR_LOG_WARNING, "CreateHome: error closing '%s': %s", dst,
-      strerror(errno));
-
+  pr_log_debug(DEBUG3, "home directory '%s' created", path);
   return 0;
 }
 
@@ -163,8 +116,8 @@ static int copy_symlink(pool *p, const char *src_dir, const char *src_path,
   int len;
 
   if ((len = pr_fsio_readlink(src_path, link_path, sizeof(link_path)-1)) < 0) {
-    log_pri(PR_LOG_WARNING, "CreateHome: error reading link '%s': %s", src_path,
-      strerror(errno));
+    pr_log_pri(PR_LOG_WARNING, "CreateHome: error reading link '%s': %s",
+      src_path, strerror(errno));
     return -1;
   }
   link_path[len] = '\0';
@@ -176,14 +129,14 @@ static int copy_symlink(pool *p, const char *src_dir, const char *src_path,
     link_path = pdircat(p, dst_dir, link_path + strlen(src_dir), NULL);
 
   if (pr_fsio_symlink(link_path, dst_path) < 0) {
-    log_pri(PR_LOG_WARNING, "CreateHome: error symlinking '%s' to '%s': %s",
+    pr_log_pri(PR_LOG_WARNING, "CreateHome: error symlinking '%s' to '%s': %s",
       link_path, dst_path, strerror(errno));
     return -1;
   }
 
   /* Make sure the new symlink has the proper ownership. */
   if (pr_fsio_chown(dst_path, uid, gid) < 0)
-    log_pri(PR_LOG_WARNING, "CreateHome: error chown'ing '%s' to %u/%u: %s",
+    pr_log_pri(PR_LOG_WARNING, "CreateHome: error chown'ing '%s' to %u/%u: %s",
       dst_path, (unsigned int) uid, (unsigned int) gid, strerror(errno));
 
   return 0; 
@@ -198,8 +151,9 @@ static int copy_dir(pool *p, const char *src_dir, const char *dst_dir,
   DIR *dh = NULL;
   struct dirent *dent = NULL;
 
-  if ((dh = opendir(src_dir)) == NULL) {
-    log_pri(PR_LOG_WARNING, "CreateHome: error copying '%s' skel files: %s",
+  dh = opendir(src_dir);
+  if (dh == NULL) {
+    pr_log_pri(PR_LOG_WARNING, "CreateHome: error copying '%s' skel files: %s",
       src_dir, strerror(errno));
     return -1;
   }
@@ -216,7 +170,7 @@ static int copy_dir(pool *p, const char *src_dir, const char *dst_dir,
       continue;
 
     if (pr_fsio_lstat(src_path, &st) < 0) {
-      log_debug(DEBUG3, "CreateHome: unable to stat '%s' (%s), skipping",
+      pr_log_debug(DEBUG3, "CreateHome: unable to stat '%s' (%s), skipping",
         src_path, strerror(errno));
       continue;
     }
@@ -241,7 +195,17 @@ static int copy_dir(pool *p, const char *src_dir, const char *dst_dir,
       if (dst_mode & S_ISGID)
         dst_mode &= ~S_ISGID;
 
-      copy_file(p, src_path, dst_path, uid, gid, dst_mode);
+      (void) pr_fs_copy_file(src_path, dst_path);
+
+      /* Make sure the destination file has the proper ownership and mode. */
+      if (pr_fsio_chown(dst_path, uid, gid) < 0)
+        pr_log_pri(PR_LOG_WARNING, "CreateHome: error chown'ing '%s' "
+          "to %u/%u: %s", dst_path, (unsigned int) uid, (unsigned int) gid,
+          strerror(errno));
+
+      if (pr_fsio_chmod(dst_path, dst_mode) < 0)
+        pr_log_pri(PR_LOG_WARNING, "CreateHome: error chmod'ing '%s' to "
+          "%04o: %s", dst_path, (unsigned int) dst_mode, strerror(errno));
 
       continue;
 
@@ -253,7 +217,7 @@ static int copy_dir(pool *p, const char *src_dir, const char *dst_dir,
 
     /* All other file types are skipped */
     } else {
-      log_debug(DEBUG3, "CreateHome: skipping skel file '%s'", src_path);
+      pr_log_debug(DEBUG3, "CreateHome: skipping skel file '%s'", src_path);
       continue;
     }
   }
@@ -267,6 +231,7 @@ static int copy_dir(pool *p, const char *src_dir, const char *dst_dir,
  */
 int create_home(pool *p, const char *home, const char *user, uid_t uid,
     gid_t gid) {
+  int res;
   config_rec *c = find_config(main_server->conf, CONF_PARAM, "CreateHome",
     FALSE);
 
@@ -276,23 +241,25 @@ int create_home(pool *p, const char *home, const char *user, uid_t uid,
   PRIVS_ROOT
 
   /* Create the configured path. */
-  if (create_path(p, home, user, uid, gid, *((mode_t *) c->argv[2]),
-      *((mode_t *) c->argv[1])) < 0) {
+  res = create_path(p, home, user, uid, gid, *((mode_t *) c->argv[2]),
+    *((mode_t *) c->argv[1]));
+
+  if (res < 0 && errno != EEXIST) {
     PRIVS_RELINQUISH
     return -1;
   }
 
-  if (c->argv[3]) {
+  if (res == 0 && c->argv[3]) {
     char *skel_dir = c->argv[3];
 
     /* Populate the home directory with files from the configured
      * skeleton (a al /etc/skel) directory.
      */
 
-    log_debug(DEBUG4, "CreateHome: copying skel files from '%s' into '%s'",
+    pr_log_debug(DEBUG4, "CreateHome: copying skel files from '%s' into '%s'",
       skel_dir, home);
     if (copy_dir(p, skel_dir, home, uid, gid) < 0)
-      log_debug(DEBUG4, "CreateHome: error copying skel files");
+      pr_log_debug(DEBUG4, "CreateHome: error copying skel files");
   }
 
   PRIVS_RELINQUISH
