@@ -23,7 +23,7 @@
  */
 
 /* Network address routines
- * $Id: netaddr.c,v 1.33 2003/10/17 15:39:27 castaglia Exp $
+ * $Id: netaddr.c,v 1.44 2004/06/30 02:42:13 castaglia Exp $
  */
 
 #include "conf.h"
@@ -36,8 +36,195 @@
 /* Do reverse DNS lookups? */
 static int reverse_dns = 1;
 
+/* Provide replacements for needed functions. */
+
+#if !defined(HAVE_GETNAMEINFO) || defined(USE_GETNAMEINFO)
+int pr_getnameinfo(const struct sockaddr *sa, socklen_t salen, char *host,
+    size_t hostlen, char *serv, size_t servlen, int flags) {
+
+  struct sockaddr_in *sai = (struct sockaddr_in *) sa;
+
+  if (!sai || sai->sin_family != AF_INET)
+    return EAI_FAMILY;
+
+  if (serv != NULL && servlen > (size_t) 1)
+    snprintf(serv, servlen, "%lu", (unsigned long) ntohs(sai->sin_port));
+
+  if (host != NULL && hostlen > (size_t) 1) {
+    struct hostent *he = NULL;
+
+    if ((flags & NI_NUMERICHOST) == 0 &&
+        (he = gethostbyaddr((const char *) &(sai->sin_addr),
+          sizeof(sai->sin_addr), AF_INET)) != NULL &&
+        he->h_name != NULL &&
+        *he->h_name != 0) {
+
+      if (strlen(he->h_name) >= hostlen)
+          goto handle_numeric_ip;
+      sstrncpy(host, he->h_name, hostlen);
+
+    } else {
+      char *ipstr = NULL;
+
+      handle_numeric_ip:
+      ipstr = inet_ntoa(sai->sin_addr);
+      if (ipstr == NULL)
+        return EAI_SYSTEM;
+
+      if (strlen(ipstr) >= hostlen)
+        return EAI_FAIL;
+
+      sstrncpy(host, ipstr, hostlen);
+    }
+  }
+
+  return 0;
+}
+#endif /* HAVE_GETNAMEINFO or USE_GETNAMEINFO */
+
+#if !defined(HAVE_GETADDRINFO) || defined(USE_GETADDRINFO)
+int pr_getaddrinfo(const char *node, const char *service,
+    const struct addrinfo *hints, struct addrinfo **res) {
+
+  struct addrinfo *ans = NULL;
+  struct sockaddr_in *saddr = NULL;
+  const char *proto_name = "tcp";
+  int socktype = SOCK_STREAM;
+  unsigned short port = 0;
+
+  if (!res)
+    return EAI_FAIL;
+  *res = NULL;
+
+  ans = malloc(sizeof(struct addrinfo));
+  if (ans == NULL)
+    return EAI_MEMORY;
+
+  saddr = malloc(sizeof(struct sockaddr_in));
+  if (saddr == NULL) {
+    free(ans);
+    return EAI_MEMORY;
+  }
+
+  ans->ai_family = AF_INET;
+  ans->ai_addrlen = sizeof *saddr;
+  ans->ai_addr = (struct sockaddr *) saddr;
+  ans->ai_next = NULL;
+  memset(saddr, 0, sizeof(*saddr));
+  saddr->sin_family = AF_INET;
+
+  if (hints != NULL) {
+    struct protoent *pe = NULL;
+
+    if ((pe = getprotobynumber(hints->ai_protocol)) != NULL &&
+         pe->p_name != NULL &&
+         *pe->p_name != 0)
+      proto_name = pe->p_name;
+
+    if (hints->ai_socktype != 0)
+      socktype = hints->ai_socktype;
+
+    else if (strcasecmp(proto_name, "udp") == 0)
+      socktype = SOCK_DGRAM;
+  }
+
+  if (service != NULL) {
+    struct servent *se = NULL;
+
+    if ((se = getservbyname(service, proto_name)) != NULL &&
+        se->s_port > 0)
+      port = se->s_port;
+
+    else if ((port = (unsigned short) strtoul(service, NULL, 0)) <= 0 ||
+        port > 65535)
+      port = 0;
+  }
+
+  if (hints != NULL &&
+      (hints->ai_flags & AI_PASSIVE) != 0)
+    saddr->sin_addr.s_addr = htonl(INADDR_ANY);
+
+  if (node != NULL) {
+    struct hostent *he = NULL;
+
+    if ((he = gethostbyname(node)) != NULL &&
+         he->h_addr_list != NULL &&
+         he->h_addr_list[0] != NULL &&
+         he->h_length > 0 &&
+         he->h_length <= (int) sizeof(saddr->sin_addr))
+      memcpy(&saddr->sin_addr, he->h_addr_list[0], he->h_length);
+  }
+
+  ans->ai_socktype = socktype;
+  saddr->sin_port = htons(port);
+  *res = ans;
+
+  return 0;
+}
+
+void pr_freeaddrinfo(struct addrinfo *ai) {
+  if (!ai)
+    return;
+
+  if (ai->ai_addr != NULL) {
+    free(ai->ai_addr);
+    ai->ai_addr = NULL;
+  }
+  free(ai);
+}
+#endif /* HAVE_GETADDRINFO or USE_GETADDRINFO */
+
+#if !defined(HAVE_INET_NTOP)
+const char *pr_inet_ntop(int af, const void *src, char *dst, size_t len) {
+  char *res;
+
+  if (af != AF_INET) {
+    errno = EAFNOSUPPORT;
+    return NULL;
+  }
+
+  res = inet_ntoa(*((struct in_addr *) src));
+  if (res == NULL)
+    return NULL;
+
+  memcpy(dst, res, len);
+  return dst;
+}
+#endif /* !HAVE_INET_NTOP */
+
+#if !defined(HAVE_INET_PTON)
+int pr_inet_pton(int af, const char *src, void *dst) {
+  unsigned long res;
+
+  if (af != AF_INET) {
+    errno = EAFNOSUPPORT;
+    return -1;
+  }
+
+  /* inet_aton(3) would be better. However, it is not ubiquitous.  */
+  res = inet_addr(src);
+  if (res == INADDR_NONE ||
+      res == 0)
+    return 0;
+
+  memcpy(dst, &res, sizeof(res));
+  return 1;
+}
+#endif /* !HAVE_INET_PTON */
+
 #ifdef HAVE_GETHOSTBYNAME2
 static void *get_v4inaddr(pr_netaddr_t *na) {
+
+  /* This function is specifically for IPv4 clients (when gethostbyname2(2) is
+   * present) that have an IPv4-mapped IPv6 address, when performing reverse
+   * DNS checks.  This function is called iff the given netaddr object is
+   * indeed an IPv4-mapped IPv6 address.  IPv6 address have 128 bits in their
+   * sin6_addr field.  For IPv4-mapped IPv6 addresses, the relevant 32 bits
+   * are the last of those 128 bits (or, alternatively, the last 4 bytes of
+   * those 16 bytes); hence the read of 12 bytes after the start of the
+   * sin6_addr pointer.
+   */
+
   return (((char *) pr_netaddr_get_inaddr(na)) + 12);
 }
 #endif /* HAVE_GETHOSTBYNAME2 */
@@ -64,6 +251,22 @@ void pr_netaddr_clear(pr_netaddr_t *na) {
   memset(na, 0, sizeof(pr_netaddr_t));
 }
 
+pr_netaddr_t *pr_netaddr_dup(pool *p, pr_netaddr_t *na) {
+  pr_netaddr_t *dup_na;
+
+  if (!p || !na) {
+    errno = EINVAL;
+    return NULL;
+  }
+
+  dup_na = pr_netaddr_alloc(p);
+
+  pr_netaddr_set_family(dup_na, pr_netaddr_get_family(na));
+  pr_netaddr_set_sockaddr(dup_na, pr_netaddr_get_sockaddr(na));  
+
+  return dup_na;
+}
+
 pr_netaddr_t *pr_netaddr_get_addr(pool *p, const char *name,
     array_header **addrs) {
 
@@ -106,6 +309,7 @@ pr_netaddr_t *pr_netaddr_get_addr(pool *p, const char *name,
     if (addrs)
       *addrs = NULL;
 
+    pr_log_debug(DEBUG10, "'%s' resolved to an IPv6 address", name);
     return na;
   }
 #endif
@@ -124,6 +328,7 @@ pr_netaddr_t *pr_netaddr_get_addr(pool *p, const char *name,
     if (addrs)
       *addrs = NULL;
 
+    pr_log_debug(DEBUG10, "'%s' resolved to an IPv4 address", name);
     return na;
 
   } else if (res == 0) {
@@ -148,14 +353,19 @@ pr_netaddr_t *pr_netaddr_get_addr(pool *p, const char *name,
 #endif /* USE_IPV6 */
     hints.ai_socktype = SOCK_STREAM;
 
+    pr_log_debug(DEBUG10, "attempting to resolve '%s' via DNS", name);
+
     gai_res = pr_getaddrinfo(name, NULL, &hints, &info);
     if (gai_res != 0) {
-      log_pri(PR_LOG_INFO, "getaddrinfo '%s' error: %s", name,
+      pr_log_pri(PR_LOG_INFO, "getaddrinfo '%s' error: %s", name,
         res != EAI_SYSTEM ? pr_gai_strerror(gai_res) : strerror(errno));
       return NULL;
     }
 
     if (info) {
+      pr_log_debug(DEBUG10, "resolved '%s' to an %s address", name,
+        info->ai_family == AF_INET ? "IPv4" : "IPv6");
+
       /* Copy the first returned addr into na, as the return value. */
       pr_netaddr_set_family(na, info->ai_family);
       pr_netaddr_set_sockaddr(na, info->ai_addr);
@@ -171,7 +381,7 @@ pr_netaddr_t *pr_netaddr_get_addr(pool *p, const char *name,
         for (ai = info->ai_next; ai; ai = ai->ai_next) {
           pr_netaddr_t **elt = push_array(*addrs);
 
-          *elt = palloc(p, sizeof(pr_netaddr_t));
+          *elt = pcalloc(p, sizeof(pr_netaddr_t));
           pr_netaddr_set_family(*elt, ai->ai_family);
           pr_netaddr_set_sockaddr(*elt, ai->ai_addr);
         }
@@ -182,6 +392,7 @@ pr_netaddr_t *pr_netaddr_get_addr(pool *p, const char *name,
     }
   }
 
+  pr_log_debug(DEBUG10, "failed to resolve '%s' to an IP address", name);
   return NULL;
 }
 
@@ -508,11 +719,11 @@ int pr_netaddr_ncmp(const pr_netaddr_t *na1, const pr_netaddr_t *na2,
 
   /* Next, compare the remaining bits in the addresses. */
   if (nbits > 0) {
-    unsigned int mask;
+    unsigned char mask;
 
     /* Get the bytes in the addresses that have not yet been compared. */
-    unsigned int in1byte = in1[nbytes];
-    unsigned int in2byte = in2[nbytes];
+    unsigned char in1byte = in1[nbytes];
+    unsigned char in2byte = in2[nbytes];
 
     /* Build up a mask covering the bits left to be checked. */
     mask = (0xff << (8 - nbits)) & 0xff;
@@ -530,30 +741,39 @@ int pr_netaddr_ncmp(const pr_netaddr_t *na1, const pr_netaddr_t *na2,
   return 0;
 }
 
-int pr_netaddr_fnmatch(pr_netaddr_t *na, const char *pattern) {
+int pr_netaddr_fnmatch(pr_netaddr_t *na, const char *pattern, int flags) {
 
-  /* NOTE: I'm still not sure why proftpd bundles an fnmatch(3)
+  /* Note: I'm still not sure why proftpd bundles an fnmatch(3)
    * implementation rather than using the system library's implementation.
    * Needs looking into.
    *
    * The FNM_CASEFOLD flag is a GNU extension; perhaps the bundled
    * implementation was added to make that flag available on other platforms.
    */
-  int flags = PR_FNM_NOESCAPE|PR_FNM_CASEFOLD;
-  const char *dnsstr, *ipstr;
+  int match_flags = PR_FNM_NOESCAPE|PR_FNM_CASEFOLD;
 
   if (!na || !pattern) {
     errno = EINVAL;
     return -1;
   }
 
-  dnsstr = pr_netaddr_get_dnsstr(na);
-  if (pr_fnmatch(pattern, dnsstr, flags) == 0)
-    return TRUE;
+  if (flags & PR_NETADDR_MATCH_DNS) {
+    const char *dnsstr = pr_netaddr_get_dnsstr(na);
 
-  ipstr = pr_netaddr_get_ipstr(na);
-  if (pr_fnmatch(pattern, ipstr, flags) == 0)
-    return TRUE;
+    pr_log_debug(DEBUG6, "comparing DNS name '%s' to pattern '%s'", dnsstr,
+      pattern);
+    if (pr_fnmatch(pattern, dnsstr, match_flags) == 0)
+      return TRUE;
+  }
+
+  if (flags & PR_NETADDR_MATCH_IP) {
+    const char *ipstr = pr_netaddr_get_ipstr(na);
+
+    pr_log_debug(DEBUG6, "comparing IP address '%s' to pattern '%s'", ipstr,
+      pattern);
+    if (pr_fnmatch(pattern, ipstr, match_flags) == 0)
+      return TRUE;
+  }
 
   return FALSE;
 }
@@ -582,7 +802,7 @@ const char *pr_netaddr_get_ipstr(pr_netaddr_t *na) {
     pr_netaddr_get_sockaddr_len(na), buf, sizeof(buf), NULL, 0, NI_NUMERICHOST);
 
   if (res != 0) {
-    log_pri(PR_LOG_NOTICE, "getnameinfo error: %s",
+    pr_log_pri(PR_LOG_NOTICE, "getnameinfo error: %s",
       res != EAI_SYSTEM ? pr_gai_strerror(res) : strerror(errno));
     return NULL;
   }
@@ -671,7 +891,7 @@ const char *pr_netaddr_get_dnsstr(pr_netaddr_t *na) {
         name = ok ? buf : NULL;
 
       } else
-        log_debug(DEBUG1, "notice: unable to resolve '%s': %s", buf,
+        pr_log_debug(DEBUG1, "notice: unable to resolve '%s': %s", buf,
           hstrerror(errno));
     }
   }
