@@ -23,7 +23,7 @@
  */
 
 /* NetIO routines
- * $Id: netio.c,v 1.12 2003/02/25 18:56:41 castaglia Exp $
+ * $Id: netio.c,v 1.20 2004/06/15 16:45:21 castaglia Exp $
  */
 
 #include "conf.h"
@@ -419,6 +419,7 @@ int pr_netio_poll(pr_netio_stream_t *nstrm) {
           }
 
 	  /* Otherwise, restart the call */
+          pr_signals_handle();
           continue;
         }
 
@@ -426,17 +427,17 @@ int pr_netio_poll(pr_netio_stream_t *nstrm) {
         nstrm->strm_errno = errno;
         return -1;
 
-    case 0:
-      /* In case the kernel doesn't support interrupted syscalls. */
-      if (nstrm->strm_flags & PR_NETIO_SESS_ABORT) {
-        nstrm->strm_flags &= ~PR_NETIO_SESS_ABORT;
-        return 1;
-      }
+      case 0:
+        /* In case the kernel doesn't support interrupted syscalls. */
+        if (nstrm->strm_flags & PR_NETIO_SESS_ABORT) {
+          nstrm->strm_flags &= ~PR_NETIO_SESS_ABORT;
+          return 1;
+        }
 
-      continue;
+        continue;
 
-    default:
-      return 0;
+      default:
+        return 0;
     }
   }
 
@@ -468,7 +469,7 @@ int pr_netio_postopen(pr_netio_stream_t *nstrm) {
 
 int pr_netio_printf(pr_netio_stream_t *nstrm, const char *fmt, ...) {
   va_list msg;
-  char buf[PR_TUNABLE_BUFFER_SIZE] = {'\0'};
+  char buf[PR_RESPONSE_BUFFER_SIZE] = {'\0'};
 
   if (!nstrm) {
     errno = EINVAL;
@@ -485,7 +486,7 @@ int pr_netio_printf(pr_netio_stream_t *nstrm, const char *fmt, ...) {
 
 int pr_netio_printf_async(pr_netio_stream_t *nstrm, char *fmt, ...) {
   va_list msg;
-  char buf[PR_TUNABLE_BUFFER_SIZE] = {'\0'};
+  char buf[PR_RESPONSE_BUFFER_SIZE] = {'\0'};
 
   if (!nstrm) {
     errno = EINVAL;
@@ -524,9 +525,9 @@ int pr_netio_write(pr_netio_stream_t *nstrm, char *buf, size_t buflen) {
         return -1;
 
       default:
-        /* We have to potentially restart here as well, in case we get EINTR.
-         */
+        /* We have to potentially restart here as well, in case we get EINTR. */
         do {
+          pr_signals_handle(); 
           run_schedule();
 
           switch (nstrm->strm_type) {
@@ -587,6 +588,7 @@ int pr_netio_write_async(pr_netio_stream_t *nstrm, char *buf, size_t buflen) {
 
   while (buflen) {
     do {
+      pr_signals_handle();
 
       switch (nstrm->strm_type) {
         case PR_NETIO_STRM_CTRL:
@@ -649,6 +651,7 @@ int pr_netio_read(pr_netio_stream_t *nstrm, char *buf, size_t buflen,
     bufmin = buflen;
 
   while (bufmin > 0) {
+    polling:
     switch (pr_netio_poll(nstrm)) {
       case 1:
         return -2;
@@ -658,6 +661,7 @@ int pr_netio_read(pr_netio_stream_t *nstrm, char *buf, size_t buflen,
 
       default:
         do {
+          pr_signals_handle();
           run_schedule();
 
           switch (nstrm->strm_type) {
@@ -678,12 +682,11 @@ int pr_netio_read(pr_netio_stream_t *nstrm, char *buf, size_t buflen,
           }
 
 #ifdef EAGAIN
-          /* Treat EAGAIN as EINTR */
-	  if (errno == EAGAIN)
-            errno = EINTR;
+	  if (bread == -1 && errno == EAGAIN)
+            goto polling;
 #endif
 
-        } while (bread == -1 && (errno == EINTR));
+        } while (bread == -1 && errno == EINTR);
         break;
     }
 
@@ -718,21 +721,18 @@ int pr_netio_shutdown(pr_netio_stream_t *nstrm, int how) {
   if (nstrm->strm_type == PR_NETIO_STRM_CTRL) {
     res = ctrl_netio ? ctrl_netio->shutdown(nstrm, how) :
       core_ctrl_netio->shutdown(nstrm, how);
-    destroy_pool(nstrm->strm_pool);
     return res;
   }
 
   if (nstrm->strm_type == PR_NETIO_STRM_DATA) {
     res = data_netio ? data_netio->shutdown(nstrm, how) :
       core_data_netio->shutdown(nstrm, how);
-    destroy_pool(nstrm->strm_pool);
     return res;
   }
 
   if (nstrm->strm_type == PR_NETIO_STRM_OTHR) {
     res = othr_netio ? othr_netio->shutdown(nstrm, how) :
       core_othr_netio->shutdown(nstrm, how);
-    destroy_pool(nstrm->strm_pool);
     return res;
   }
 
@@ -841,7 +841,7 @@ char *pr_netio_telnet_gets(char *buf, size_t buflen,
     } else
       toread = pbuf->buflen - pbuf->remaining;
 
-    while (buflen && *pbuf->current != '\n' && toread--) {
+    while (buflen && toread > 0 && *pbuf->current != '\n' && toread--) {
       cp = *pbuf->current++;
       pbuf->remaining++;
 
@@ -994,7 +994,7 @@ pr_netio_t *pr_alloc_netio(pool *parent_pool) {
   return netio;
 }
 
-void pr_init_netio(void) {
+void init_netio(void) {
   signal(SIGPIPE, SIG_IGN);
   signal(SIGURG, SIG_IGN);
 
