@@ -26,7 +26,7 @@
 /* Shows who is online via proftpd, in a manner similar to top.  Uses the
  * scoreboard files.
  *
- * $Id: ftptop.c,v 1.21 2003/02/24 18:39:52 castaglia Exp $
+ * $Id: ftptop.c,v 1.29 2004/03/05 03:24:36 castaglia Exp $
  */
 
 #define FTPTOP_VERSION "ftptop/0.9"
@@ -37,19 +37,16 @@
 #include <signal.h>
 #include <stdlib.h>
 #include <time.h>
-#include <sys/types.h>
 
 static const char *program = "ftptop";
 
-#if defined(HAVE_NCURSES_H) && defined(HAVE_LIBNCURSES)
-#define HAVE_NCURSES 1
-#include <ncurses.h>
-#endif /* HAVE_NCURSES_H */
-
-/* We don't want to include both ncurses.h and curses.h. */
-#if defined(HAVE_CURSES_H) && defined(HAVE_LIBCURSES) && !defined(HAVE_NCURSES_H)
-#define HAVE_CURSES 1
+/* ncurses is preferred...*/
 
+#if defined(HAVE_NCURSES_H) && defined(HAVE_LIBNCURSES) && defined(USE_NCURSES)
+# define HAVE_NCURSES 1
+# include <ncurses.h>
+#elif defined(HAVE_CURSES_H) && defined(HAVE_LIBCURSES) && defined(USE_CURSES)
+# define HAVE_CURSES 1
 /* Sigh...portability.  It seems that Solaris' curses.h (at least for 2.8)
  * steps on wide-character macros, generating compiler warnings.  This, then
  * is just a hack to silence the compiler.
@@ -58,19 +55,28 @@ static const char *program = "ftptop";
 #  define __lint
 # endif
 # include <curses.h>
-#endif /* HAVE_CURSES_H */
+#endif
 
 #if defined(HAVE_NCURSES) || defined(HAVE_CURSES)
 
 /* Display options */
 
-/* These are for displaying "PID S USER ADDR SERVER TIME COMMAND" */
-#define FTPTOP_REG_HEADER_FMT	"%-5s %s %-8s %-20s %-15s %-4s %-20s\n"
-#define FTPTOP_REG_DISPLAY_FMT	"%-5u %s %-8.8s %-20.20s %-15s %lu %-20.20s\n"
+/* These are for displaying "PID S USER CLIENT SERVER TIME COMMAND" */
+#define FTPTOP_REG_HEADER_FMT	"%-5s %s %-8s %-20s %-15s %-4s %-*s\n"
+#define FTPTOP_REG_DISPLAY_FMT	"%-5u %s %-8.8s %-20.20s %-15s %-6.6s %4s %-*.*s\n"
 
-/* These are for displaying tranfer data: "PID S USER ADDR KB/s %DONE" */
-#define FTPTOP_XFER_HEADER_FMT	"%-5s %s %-8s %-20s %-5s    %-32s\n"
-#define FTPTOP_XFER_DISPLAY_FMT	"%-5u %s %-8.8s %-20.20s %-3.2f %-32.20s\n"
+/* These are for displaying tranfer data: "PID S USER CLIENT KB/s %DONE" */
+#define FTPTOP_XFER_HEADER_FMT	"%-5s %s %-8s %-20s %-5s    %-*s\n"
+#define FTPTOP_XFER_DISPLAY_FMT	"%-5u %s %-8.8s %-46.46s %-3.2f %6s%%s\n"
+
+#define FTPTOP_REG_ARG_MIN_SIZE		20
+#define FTPTOP_XFER_DONE_MIN_SIZE	6
+#define FTPTOP_REG_ARG_SIZE	\
+  (COLS - (80 - FTPTOP_REG_ARG_MIN_SIZE) < FTPTOP_REG_ARG_MIN_SIZE ? \
+  FTPTOP_REG_ARG_MIN_SIZE : COLS - (80 - FTPTOP_REG_ARG_MIN_SIZE))
+#define FTPTOP_XFER_DONE_SIZE 	\
+  (COLS - (80 - FTPTOP_XFER_DONE_MIN_SIZE) < FTPTOP_XFER_DONE_MIN_SIZE ? \
+  FTPTOP_XFER_DONE_MIN_SIZE : COLS - (80 - FTPTOP_XFER_DONE_MIN_SIZE))
 
 #define FTPTOP_SHOW_DOWNLOAD		0x0001
 #define FTPTOP_SHOW_UPLOAD		0x0002
@@ -85,10 +91,12 @@ static unsigned int display_mode = FTPTOP_SHOW_REG;
 static char *config_filename = CONFIG_FILE_PATH;
 
 /* Scoreboard variables */
+static time_t ftp_uptime = 0;
 static unsigned int ftp_nsessions = 0;
 static unsigned int ftp_nuploads = 0;
 static unsigned int ftp_ndownloads = 0;
 static unsigned int ftp_nidles = 0;
+static char *server_name = NULL;
 static char **ftp_sessions = NULL;
 static unsigned int chunklen = 3;
 
@@ -97,6 +105,7 @@ static void scoreboard_close(void);
 static int scoreboard_open(void);
 
 static void show_version(void);
+static const char *show_ftpd_uptime(void);
 static void usage(void);
 
 static void clear_counters(void) {
@@ -145,6 +154,27 @@ static char *calc_percent_done(off_t size, off_t done) {
   return sbuf;
 }
 
+/* Borrowed from ftpwho.c */
+static const char *show_time(time_t *i) {
+  time_t now = time(NULL);
+  unsigned long l;
+  static char sbuf[7];
+
+  if (!i || !*i)
+    return "-";
+
+  memset(sbuf, '\0', sizeof(sbuf));
+  l = now - *i;
+
+  if (l < 3600)
+    snprintf(sbuf, sizeof(sbuf), "%lum%lus",(l / 60),(l % 60));
+  else
+    snprintf(sbuf, sizeof(sbuf), "%luh%lum",(l / 3600),
+    ((l - (l / 3600) * 3600) / 60));
+
+  return sbuf;
+}
+
 static int check_scoreboard_file(void) {
   struct stat sbuf;
 
@@ -154,6 +184,41 @@ static int check_scoreboard_file(void) {
   return 0;
 }
 
+static const char *show_ftpd_uptime(void) {
+  static char buf[128] = {'\0'};
+  time_t uptime_secs = time(NULL) - ftp_uptime;
+  int upminutes, uphours, updays;
+  int pos = 0;
+
+  if (!ftp_uptime)
+    return "";
+
+  memset(buf, '\0', sizeof(buf));
+
+  strcat(buf, ", up for ");
+  pos += strlen(buf); 
+
+  updays = (int) uptime_secs / (60 * 60 * 24);
+
+  if (updays)
+    pos += sprintf(buf + pos, "%d day%s, ", updays, (updays != 1) ? "s" : "");
+
+  upminutes = (int) uptime_secs / 60;
+
+  uphours = upminutes / 60;
+  uphours = uphours % 24;
+
+  upminutes = upminutes % 60;
+
+  if (uphours)
+    pos += sprintf(buf + pos, "%2d hr%s %02d min", uphours,
+      (uphours != 1) ? "s" : "", upminutes);
+  else
+    pos += sprintf(buf + pos, "%d min", upminutes);
+
+  return buf;
+}
+
 /* scan_config_file() is a kludge for 1.2 which does a very simplistic attempt
  * at determining what the "ScoreboardFile" directive is set to.  It will be
  * replaced in 1.3 with the abstracted configure system (hopefully).
@@ -227,7 +292,7 @@ static void scan_config_file(void) {
 
 static void process_opts(int argc, char *argv[]) {
   int optc = 0;
-  const char *prgopts = "Dd:f:hIiUV";
+  const char *prgopts = "DS:d:f:hIiUV";
 
   while ((optc = getopt(argc, argv, prgopts)) != -1) {
     switch (optc) {
@@ -236,6 +301,10 @@ static void process_opts(int argc, char *argv[]) {
         display_mode |= FTPTOP_SHOW_DOWNLOAD;
         break;
 
+      case 'S':
+        server_name = strdup(optarg);
+        break;
+
       case 'd':
         delay = atoi(optarg);
 
@@ -316,11 +385,15 @@ static void read_scoreboard(void) {
     /* Default status: "A" for "authenticating" */
     char *status = "A";
 
+    /* If a ServerName was given, skip unless the scoreboard entry matches. */
+    if (server_name && strcmp(server_name, score->sce_server_label) != 0)
+      continue;
+
     /* Clear the buffer for this run. */
     memset(buf, '\0', sizeof(buf));
 
     /* Determine the status symbol to display. */
-    if (strstr(score->sce_cmd, "(idle)")) {
+    if (strcmp(score->sce_cmd, "idle") == 0) {
       status = "I";
       ftp_nidles++;
 
@@ -328,7 +401,7 @@ static void read_scoreboard(void) {
           !(display_mode & FTPTOP_SHOW_IDLE))
         continue;
 
-    } else if (strstr(score->sce_cmd, "RETR")) {
+    } else if (strcmp(score->sce_cmd, "RETR") == 0) {
       status = "D";
       ftp_ndownloads++;
 
@@ -336,9 +409,9 @@ static void read_scoreboard(void) {
           !(display_mode & FTPTOP_SHOW_DOWNLOAD))
         continue;
 
-    } else if (strstr(score->sce_cmd, "STOR") ||
-        strstr(score->sce_cmd, "APPE") ||
-        strstr(score->sce_cmd, "STOU")) {
+    } else if (strcmp(score->sce_cmd, "STOR") == 0 ||
+        strcmp(score->sce_cmd, "APPE") == 0 ||
+        strcmp(score->sce_cmd, "STOU") == 0) {
       status = "U";
       ftp_nuploads++;
 
@@ -346,15 +419,16 @@ static void read_scoreboard(void) {
           !(display_mode & FTPTOP_SHOW_UPLOAD))
         continue;
 
-    } else if (strstr(score->sce_cmd, "LIST") ||
-        strstr(score->sce_cmd, "NLST"))
+    } else if (strcmp(score->sce_cmd, "LIST") == 0 ||
+        strcmp(score->sce_cmd, "NLST") == 0)
       status = "L";
 
     if (display_mode != FTPTOP_SHOW_RATES) {
       snprintf(buf, sizeof(buf), FTPTOP_REG_DISPLAY_FMT,
         (unsigned int) score->sce_pid, status, score->sce_user,
         score->sce_client_name, score->sce_server_addr,
-        time(NULL) - score->sce_begin_session, score->sce_cmd);
+        show_time(&score->sce_begin_session), score->sce_cmd,
+       FTPTOP_REG_ARG_SIZE, FTPTOP_REG_ARG_SIZE, score->sce_cmd_arg);
       buf[sizeof(buf)-1] = '\0';
 
     } else {
@@ -397,42 +471,56 @@ static void scoreboard_close(void) {
 static int scoreboard_open(void) {
   int res = 0;
 
-  if ((res = util_open_scoreboard(O_RDONLY, NULL)) < 0) {
+  if ((res = util_open_scoreboard(O_RDONLY)) < 0) {
     switch (res) {
-      case -1:
-        fprintf(stderr, "%s: unable to open scoreboard: %s\n", program,
-          strerror(errno));
-        return res;
-
       case UTIL_SCORE_ERR_BAD_MAGIC:
-        fprintf(stderr, "%s: scoreboard is corrupted or old\n", program);
+        fprintf(stderr, "%s: error opening scoreboard: bad/corrupted file\n",
+          program);
         return res;
 
       case UTIL_SCORE_ERR_OLDER_VERSION:
-        fprintf(stderr, "%s: scoreboard is too old\n", program);
+        fprintf(stderr, "%s: error opening scoreboard: bad version (too old)\n",
+          program);
         return res;
 
       case UTIL_SCORE_ERR_NEWER_VERSION:
-        fprintf(stderr, "%s: scoreboard is too new\n", program);
+        fprintf(stderr, "%s: error opening scoreboard: bad version (too new)\n",
+          program);
+        return res;
+
+      default:
+        fprintf(stderr, "%s: error opening scoreboard: %s\n",
+          program, strerror(errno));
         return res;
     }
   }
 
+  ftp_uptime = util_scoreboard_get_daemon_uptime();
+
   return 0;
 }
 
 static void show_sessions(void) {
   time_t now;
+  char *now_str = NULL;
+  const char *uptime_str = NULL;
 
   clear_counters();
   read_scoreboard();
 
   time(&now);
+
+  /* Trim ctime(3)'s trailing newline. */
+  now_str = ctime(&now);
+  now_str[strlen(now_str)-1] = '\0';
+
+  uptime_str = show_ftpd_uptime();
+
   wclear(stdscr);
   move(0, 0);
 
   attron(A_BOLD);
-  printw(FTPTOP_VERSION ": %s", ctime(&now));
+  printw(FTPTOP_VERSION ": %s%s\n", now_str, uptime_str);
   printw("%u Total FTP Sessions: %u downloading, %u uploading, %u idle\n",
     ftp_nsessions, ftp_ndownloads, ftp_nuploads, ftp_nidles);
   attroff(A_BOLD);
@@ -442,11 +530,10 @@ static void show_sessions(void) {
   attron(A_REVERSE);
 
   if (display_mode != FTPTOP_SHOW_RATES)
-    printw(FTPTOP_REG_HEADER_FMT, "PID", "S", "USER", "ADDR", "SERVER",
-      "TIME", "COMMAND");
-
+    printw(FTPTOP_REG_HEADER_FMT, "PID", "S", "USER", "CLIENT", "SERVER",
+      "TIME", FTPTOP_REG_ARG_SIZE, "COMMAND");
   else
-    printw(FTPTOP_XFER_HEADER_FMT, "PID", "S", "USER", "ADDR", "KB/s", "%DONE");
+    printw(FTPTOP_XFER_HEADER_FMT, "PID", "S", "USER", "CLIENT", "KB/s", FTPTOP_XFER_DONE_SIZE, "%DONE");
 
   attroff(A_REVERSE);
 
